/***********************************************************************
**                                                                    **
** CEMoS Version 4.1.2 Copyright(C) 2023 Frank Hamberg, Cora Kohlmeier**
** last changes 21/11/01 CK                                           **
**                                                                    **
***********************************************************************/
void pass0(void)
{
/*
** In pass0 werden die Anzahlen der Variablen vom Typ Matrix bestimmt und
** die Namen ermittelt und die Pointer fuer die namensgebundenen Zugriffe
** deklariert.
** Weiterhin werden die Parameter mit vorgebener Struktur schon in die
** Zieldateien uebertragen.
*/
/* 0. Fall Konsitenzprüfung fuer 0D-Typen */
if (type >= _0D_STATE && type < STATE)
 {
 bp1=strpos(input, "[");
 bp2=strpos(input, "]");
 if (bp2>0 || bp1>0)
 {
     printf("%s \n ERROR: Scalar type defined, but indexed type given\n",input);
     exit (1);
 }
 bp1=strpos(input, "{");
 bp2=strpos(input, "}");
 if (bp2>0 || bp1>0)
 {
     printf("%s \n ERROR: Scalar type defined, but indexed type given\n",input);
     exit (1);
 }
 else bp1=strpos(input,"=");
 if (bp1<0) bp1=strpos(input,";");

 if (bp1>0) sprintf(svar,"%s",strmid(input,0,bp1));
    else sprintf(svar,"%s",input);

 if (strlen(svar) > sizeof(long_name)-1)
 {
     printf ("ERROR:In %s name too long: %s\n",struct_file,svar);
     exit (1);
 }

 sprintf(all_names[num_vars],"%s",svar);
 all_types[num_vars]=type;
 num_vars++;
 }
/* Ende 0. Fall Konsitenzprüfung fuer 0D-Typen */


/* 1. Fall Index-Pruefung fuer 1D-Felder */
if (type >= STATE && type <= FEATURES)
 {
 bp1=strpos(input, "[");
 bp2=strpos(input, "]");
 if (bp2>bp1)
 {
  box=atoi(strmid(input,bp1+1,bp2-bp1-1));

  if (type == STATE || type == GLOBAL || type == INT_PARAM || type == REAL_PARAM)
  {
    if (boxes == -1) boxes=box;

    if (boxes != box)
    {
     printf("%s \n ERROR: Size of vector is not consistent, expected size=%d\n",input,boxes);
     exit (1);
    }
  }
 }
 else bp1=strpos(input,"=");
 if (bp1<0) bp1=strpos(input,";");

 if (bp1>0) sprintf(svar,"%s",strmid(input,0,bp1));
    else sprintf(svar,"%s",input);

 if (strlen(svar) > sizeof(long_name)-1)
 {
     printf ("In %s name too long: %s\n",struct_file,svar);
     exit (1);
 }

 sprintf(all_names[num_vars],"%s",svar);
 all_types[num_vars]=type;
 num_vars++;
 }
/* Ende 1. Fall  Index-Pruefung fuer 1D-Felder */

/* 2. Fall  Index-Pruefung fuer 2D-Felder */
if (type >= _2D_STATE && type <= _2D_FEATURES)
 {
 bp1=numchar(input,"[");
 bp2=numchar(input,"]");
 if (bp1<=0)
 {
    bp1=strpos(input,"=");
    if (bp1<0) bp1=strpos(input,";");

    if (bp1>0) sprintf(svar,"%s",strmid(input,0,bp1));
       else sprintf(svar,"%s",input);
  }
 else
 {
  if (get_entries(box_entries,input,svar) != 2)
  {
     printf("ERROR: Wrong number of indices in statement %s\n",input);
     exit (1);
  } 
  box1 = box_entries[0];
  box2 = box_entries[1];

  fprintf(struct_def,"/*prepare->pass0:type=%d,box1=%d,box2=%d*/\n",type,box1,box2);
  if (type == _2D_STATE || type == _2D_GLOBAL || type == _2D_INT_PARAM || type == _2D_REAL_PARAM)
  {
    if (boxes1 == -1) boxes1=box1;
    if (boxes2 == -1) boxes2=box2;


    if (boxes1 != box1 || boxes2 != box2)
    {
     printf("ERROR: Size of matrix is not consistent, expected sizes=[%d][%d] got input %s\n",boxes1,boxes2,input);
     exit (1);
    }
  }
 }
 
 if (strlen(svar) > sizeof(long_name)-1)
 {
     printf ("In %s name too long: %s\n",struct_file,svar);
     exit (1);
 }

 sprintf(all_names[num_vars],"%s",svar);
 all_types[num_vars]=type;
 num_vars++;
 fprintf(struct_def,"/*prepare->pass0:input=%s,svar=%s*/\n",input,svar);
}
 
/* Ende 2. Fall  Index-Pruefung fuer 2D-Felder */

switch (type)
 {

/* 0D Faelle */
  case _0D_STATE:fprintf(struct_def,"real %s;\n",svar);
         fprintf(struct_h,"extern real %s;\n",svar);
         sprintf(_0D_state_names[num_0D_states],"%s",svar);
         num_0D_states++;
         break;

  case _0D_GLOBAL:fprintf(struct_def,"real %s;\n",svar);
         fprintf(struct_h,"extern real %s;\n",svar);
         sprintf(_0D_global_names[num_0D_globals],"%s",svar);
         num_0D_globals++;
         break;

/* 1D Faelle */
  case STATE:fprintf(struct_def,"real *%s;\n",svar);
         fprintf(struct_h,"extern real *%s;\n",svar);
         sprintf(_1D_state_names[num_1D_states],"%s",svar);
         num_1D_states++;
         break;

  case GLOBAL:fprintf(struct_def,"real *%s;\n",svar);
         fprintf(struct_h,"extern real *%s;\n",svar);
         sprintf(_1D_global_names[num_1D_globals],"%s",svar);
         num_1D_globals++;
         break;

  case INT_PARAM:fprintf(struct_def,"int *%s;\n",svar);
         fprintf(struct_h,"extern int *%s;\n",svar);
         break;

  case REAL_PARAM:fprintf(struct_def,"real *%s;\n",svar);
         fprintf(struct_h,"extern real *%s;\n",svar);
         break;

  case REAL_CONST:if (strpos(input,";")<0) strcat(input,";");
         fprintf(struct_def,"real %s \n",input);
         bp1=strpos(input,"=");
         sprintf(svar,"%s",strmid(input,0,bp1));
         fprintf(struct_h,"%s%s%s \n","extern real ",svar,";");
         break;

  case INT_CONST:if (strpos(input,";")<0) strcat(input,";");
         fprintf(struct_def,"int %s \n",input);
         bp1=strpos(input,"=");
         sprintf(svar,"%s",strmid(input,0,bp1));
         fprintf(struct_h,"%s%s%s \n","extern int ",svar,";");
         break;

  case DERIVES:sprintf(_1D_float_derivs[num_1D_float_derivs],"%s",input);
         num_1D_float_derivs++;
         break;

  case FEATURES:sprintf(_1D_int_derivs[num_1D_int_derivs],"%s",input);
         num_1D_int_derivs++;
         break;

  case FLUXES:sprintf(_1D_glob_derivs[num_1D_glob_derivs],"%s",input);
         num_1D_glob_derivs++;
         break;

/* 2D Faelle */
  case _2D_STATE:fprintf(struct_def,"real **%s;\n",svar);
         fprintf(struct_h,"extern real **%s;\n",svar);
         sprintf(_2D_state_names[num_2D_states],"%s",svar);
         num_2D_states++;
         break;

  case _2D_GLOBAL:fprintf(struct_def,"real **%s;\n",svar);
         fprintf(struct_h,"extern real **%s;\n",svar);
         sprintf(_2D_global_names[num_2D_globals],"%s",svar);
         num_2D_globals++;
         break;

  case _2D_INT_PARAM:fprintf(struct_def,"int **%s;\n",svar);
         fprintf(struct_h,"extern int **%s;\n",svar);
         break;
  case _2D_REAL_PARAM:fprintf(struct_def,"real **%s;\n",svar);
         fprintf(struct_h,"extern real **%s;\n",svar);
         break;
  case _2D_DERIVES:sprintf(_2D_float_derivs[num_2D_float_derivs],"%s",input);
         num_2D_float_derivs++;
         break;
  case _2D_FEATURES:sprintf(_2D_int_derivs[num_2D_int_derivs],"%s",input);
         num_2D_int_derivs++;
         break;
  case _2D_FLUXES:sprintf(_2D_glob_derivs[num_2D_glob_derivs],"%s",input);
         num_2D_glob_derivs++;
         break;


  default:break;
 }
} /* ende von pass0 */
 
