/***********************************************************************
**                                                                    **
** CEMoS Version 4.1.2 Copyright(C) 2023 Frank Hamberg, Cora Kohlmeier**
** last changes 2012/07/28 fh: endtime, starttime, storetime zusätzlich eingebaut für Verwendung in cemos.par und im Modell    
** 2012/10/31 fh: Korrektur in get_1D_store_ptr um _1D_glob_derivs speichern zu koennen (strlen(_1D_state_names) statt strlen(_1D_glob_derivs))
**
**                                                                    **
***********************************************************************/
#define __PREPARE__
#include "prep_types.h"
#include "prepare.h"

string struct_file,header_file,include_path,reset_derivs;
string function_name,definitions,parameter_file,model_path,dummy,definition_file;
char *input,*helpstr,*eustr, *fc;
long_name svar,new_name;

long_name *_0D_state_names,*_0D_global_names;
long_name *_1D_state_names,*_1D_global_names;
long_name *_2D_state_names,*_2D_global_names;

long_name *all_names;
long_name *_0D_float_derivs,*_0D_int_derivs,*_0D_glob_derivs;
long_name *_1D_float_derivs,*_1D_int_derivs,*_1D_glob_derivs;
long_name *_2D_float_derivs,*_2D_int_derivs,*_2D_glob_derivs;

int *all_types;
int i,j,k,l,num_vars=0;
int bp1,bp2,sb1,sb2,cp;
int type,pass,keyword,comment=0;
int num_0D_float_derivs=1,num_0D_int_derivs=0,num_0D_states=0,num_0D_globals=0,num_0D_glob_derivs=0;
int num_1D_float_derivs=1,num_1D_int_derivs=0,num_1D_states=0,num_1D_globals=0,num_1D_glob_derivs=0;
int num_2D_float_derivs=1,num_2D_int_derivs=0,num_2D_states=0,num_2D_globals=0,num_2D_glob_derivs=0;
int num_3D_float_derivs=1,num_3D_int_derivs=0,num_3D_states=0,num_3D_globals=0,num_3D_glob_derivs=0;
int get_int;
float get_flt;

int box_entries[MAX_DIMENSION];
int boxes=-1,boxes1=-1,boxes2=-1,boxes3=-1;
int box,box1,box2,box3;

FILE *global_defs;
FILE *struct_h;
FILE *struct_def;
FILE *res_derivs;

void strtrim(char*);

int main(int argc, char *argv[])
  {
  unsigned int num_changes=0;
  void pass0(void);
  void pass1(void);
  void build_structures(void);


/* Uebergebene Argumente retten */
  sprintf(definitions,"%s",argv[1]);
  sprintf(struct_file,"%s",argv[2]);
  sprintf(header_file,"%s",argv[3]);
  sprintf(function_name,"%s",argv[4]);
  sprintf(include_path,"%s",argv[5]);
  sprintf(parameter_file,"%s",argv[6]);
  sprintf(model_path,"%s",argv[7]);
  sprintf(reset_derivs,"%s",argv[8]);
  sprintf(dummy,"%s",argv[9]);
  sprintf(definition_file,"%s",argv[10]);


  all_types=(int *) malloc( total_vars*sizeof(int));
  all_names=(long_name *) malloc( total_vars*sizeof(long_name));

  _0D_state_names=(long_name *) malloc( one_var*sizeof(long_name));
  _0D_global_names=(long_name *) malloc( one_var*sizeof(long_name));
  _0D_glob_derivs=(long_name *) malloc( one_var*sizeof(long_name));
  _0D_float_derivs=(long_name *) malloc( one_var*sizeof(long_name));
  _0D_int_derivs=(long_name *) malloc( one_var*sizeof(long_name));

  _1D_state_names=(long_name *) malloc( one_var*sizeof(long_name));
  _1D_global_names=(long_name *) malloc( one_var*sizeof(long_name));
  _1D_glob_derivs=(long_name *) malloc( one_var*sizeof(long_name));
  _1D_float_derivs=(long_name *) malloc( one_var*sizeof(long_name));
  _1D_int_derivs=(long_name *) malloc( one_var*sizeof(long_name));

  _2D_state_names=(long_name *) malloc( one_var*sizeof(long_name));
  _2D_global_names=(long_name *) malloc( one_var*sizeof(long_name));
  _2D_glob_derivs=(long_name *) malloc( one_var*sizeof(long_name));
  _2D_float_derivs=(long_name *) malloc( one_var*sizeof(long_name));
  _2D_int_derivs=(long_name *) malloc( one_var*sizeof(long_name));

/* Zu den state-Variablen gibt es immer den derived-Typ S***  */
  sprintf(_0D_float_derivs[0],"%s","S+variable");
  sprintf(_1D_float_derivs[0],"%s","S+variable");
  sprintf(_2D_float_derivs[0],"%s","S+variable");

/* Files fuer Strukturdefinitionen oeffen
**         struc_c: hier werden die Strukuren vorbereitet
**         struct_h : Include file mit den globalen Deklarationen
*/

  input=(char *) malloc( 2*_BUFSIZE_*sizeof(char));
  helpstr=(char *) malloc( _BUFSIZE_*sizeof(char));

  struct_def=fopen(struct_file,"w+");
  struct_h=fopen(header_file,"w+");
  res_derivs=fopen(reset_derivs,"w+");

  fprintf(struct_def,auto_header);
  fprintf(struct_h,auto_header);
  fprintf(res_derivs,auto_header);

/* File mit den Modell setups oeffnen */

  global_defs=fopen(definitions,"rb");


/* Im ersten Durchgang wird:
**     1. Der File Header erzeugt und
**     2. die globalen Variablen deklariert
*/
  fprintf(struct_def,"#include \"stdinc.h\" \n");
  fprintf(struct_def,"#include \"constructs.h\" \n");
  fprintf(struct_def,"#include \"types.h\" \n");
  fprintf(struct_def,"shared_use_st *com_cemos;\n");
  fprintf(struct_def,"real startim,endtim,outdelt,storetim=0.;\n");
  fprintf(struct_def,"real starttime=-1.0,endtime=-1.0,storetime=0.;\n");
  
  fprintf(struct_def,"real acc,relchange,relrate,maxdelt,mindelt;\n");
  fprintf(struct_def,"real year=0.,cycle=360.,sim_time,delt;\n");
  fprintf(struct_def,"int method,assign,steps_of_int_to_go,liap=0,recalc_globals=0;\n");
  fprintf(struct_def,"unsigned int int_time,day_of_year;\n");
  fprintf(struct_def,"long nvar;\n");
  fprintf(struct_def,"int found_change;\n");
  fprintf(struct_def,"string resultname,resultdir,command;\n");

  fprintf(struct_h,"#include \"stdinc.h\" \n");
  fprintf(struct_h,"#include \"constructs.h\" \n");
  fprintf(struct_h,"#include \"types.h\" \n");
/*  fprintf(struct_h,"#include \"tools.h\" \n");*/
  fprintf(struct_h,"#define model_path \"%s\" \n",model_path);
  fprintf(struct_h,"extern shared_use_st *com_cemos;\n");
  fprintf(struct_h,"extern real startim,endtim,outdelt,storetim;\n");
  fprintf(struct_h,"extern real starttime,endtime,storetime;\n");
  fprintf(struct_h,"extern real acc,relchange,relrate,maxdelt,mindelt;\n");
  fprintf(struct_h,"extern real year,cycle,sim_time,delt;\n");
  fprintf(struct_h,"extern int method,assign,steps_of_int_to_go,liap,recalc_globals;\n");
  fprintf(struct_h,"extern unsigned int int_time,day_of_year;\n");
  fprintf(struct_h,"extern long nvar;\n");
  fprintf(struct_h,"extern string resultname,resultdir;\n");

  fprintf(struct_h,"void %s(string sensitivity);\n",function_name);


  for (pass=0;pass<=2;pass++)
    {
    comment=0;
    type=0;
    while ( fgets(input,100,global_defs) != 0 )
      {
/*Praeprozessor-"Muell"  entfernen */
      if (strpos(input,"#")>=0) sprintf(input,"%s","");

      cp=strpos(input,rem1);
      comment=cp-strpos(input,rem2);
      if (cp==0 && comment < 0) sprintf(input,"%s","");

      while (comment > 0)
        {
        if (fgets(input,100,global_defs) == 0 ) goto stop_label;
        if (strpos(input,rem2) >= 0)
          {
          comment=-1;
          if (fgets(input,100,global_defs) == 0 ) goto stop_label;
          }
        } /* ende von comment>=0 */
/* Sonderfaelle, die nicht durch check_keyword erledigt werden koennen */
      if (strpos(input,_keyword_)          >= 0) type=-1;
      if (strpos(input,_ljapunov_)   >= 0)  fprintf(struct_h,"#define __ljapunov__\n");
/*      if (strpos(input,_use_slave_)   >= 0) fprintf(struct_h,"#define __with_slave__\n"); */
      keyword = check_keyword(&type,input);

      strtrim(input);

      if (keyword==0 && type>0 && ((type>10 && strlen(input)>3) || (type<10 && strlen(input)>1)))
      {
      sb1=strpos(input,"{");
      sb2=strpos(input,"}");
      if (sb1<0 && sb2>0) 
      {
          printf("Error in %s in line containing %s. Missing or wrong delimiter.\nExiting!\n",definition_file,input);
          exit(1);
      }
      if (sb1>0)
       {
        while(sb2<0)
         {
         fc=fgets(helpstr,100,global_defs);
         strcat(input,helpstr);
         sb2=strpos(input,"}");
         if (strpos(helpstr,"{")>0 || strpos(helpstr,"{")>0 || (sb2<0 && fc==0))
          {
               printf("Error in %s in line containing %s. Missing or wrong delimiter.\nExiting!\n",definition_file,input);
              exit(1);
          }
         }
         strtrim(input);
         switch (pass)
            {
            case 0: pass0();
                          fflush(struct_def);
                          fflush(struct_h); 
                          break;
            case 1: pass1();
                          fflush(struct_def);
                          fflush(struct_h); 
                          break;
            case 2: 
            default:fflush(struct_def);
                          fflush(struct_h); 
                         break;
            }
       }
       else
       {
        sprintf(helpstr,"%s",input);
        cp=strpos(helpstr,rem1);
        if (cp>0) sprintf(helpstr,"%s",strmid(helpstr,0,cp));

        while(strlen(helpstr)>0)
         {
         cp=strpos(helpstr,",");
         if (cp>0)
          {
          sb1=strlen(helpstr);
          sprintf(input,"%s",strmid(helpstr,0,cp));
          sprintf(helpstr,"%s",strmid(helpstr,cp+1,sb1-cp-1));
          } 
          else
          {
          sprintf(input,"%s",helpstr);
          sprintf(helpstr,"%s","");
          }
         if (strpos(input,";") < strpos(input,"{"))
         {
            printf("Error in %s in line containing %s. Missing or wrong delimiter.\nExiting!\n",definition_file,input);
            exit(1);
         }
         switch (pass) 
            {
            case 0: pass0();
                          fflush(struct_def);
                          fflush(struct_h); break;
            case 1: pass1();
                          fflush(struct_def);
                          fflush(struct_h); break;
            case 2: 
            default: fflush(struct_def);
                          fflush(struct_h); 
                         break;
            }
         }
        }

       } /* ende von if (keyword ... */
    
    }  /* ende von while fscan() != EOF */

        if (pass == 0)
        {
        rewind(global_defs);


        build_structures();


        } 
        if (pass ==2) 
        {
/* Files schliessen nach dem zweiten Durchgang  */


       fclose(global_defs);



/* 
** Fuer jede Variable, ausser States, Globals und Derivs+variable wird ein
** Sucheintrag erzeugt
*/
      for (j=0;j<num_vars;j++) 
        {
        type=all_types[j];
      
        switch (type)
          {

/* 1D Faelle  */
         case STATE:
          case GLOBAL: break;

          case INT_PARAM:fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s\"); ",num_changes,"%s",all_names[j]);
                 fprintf(struct_def,"change_ptr[%d]=(real*) %s; ",num_changes,all_names[j]);
                 fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,type);
                 num_changes++;
                 break;

          case REAL_PARAM:fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s\"); ",num_changes,"%s",all_names[j]);
                 fprintf(struct_def,"change_ptr[%d]=%s; ",num_changes,all_names[j]);
                 fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,type);
                 num_changes++;
                 break;
    
          case REAL_CONST:fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s\"); ",num_changes,"%s",all_names[j]);
                 fprintf(struct_def,"change_ptr[%d]=&%s; ",num_changes,all_names[j]);
                 fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,type);
                 fprintf(struct_def,"printf(\"Initializing float parameter: %s=%s  from %s\\n\",(float) %s); \n",all_names[j],"%f",definition_file,all_names[j]);
                 num_changes++;
                 break;

          case INT_CONST:fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s\"); ",num_changes,"%s",all_names[j]);
                 fprintf(struct_def,"change_ptr[%d]=(real*) &%s; ",num_changes,all_names[j]);
                 fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,type);
                 fprintf(struct_def,"printf(\"Initializing integer parameter: %s=%s from %s\\n\",(int) %s); \n",all_names[j],"%d",definition_file,all_names[j]);
                 num_changes++;
                 break;

          case DERIVES:cp=strpos(all_names[j],"+base");
                 if (cp < 0) break;
                 sprintf(helpstr,"%s",strmid(all_names[j],0,cp));
                 for (i=0;i<num_1D_states;i++)
                   {
                   fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s%s\"); ",num_changes,"%s",helpstr,_1D_state_names[i]);
                    fprintf(struct_def,"change_ptr[%d]= %s%s; ",num_changes,helpstr,_1D_state_names[i]);
                    fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,type);
                    num_changes++;

                   }
                 break;

          case FEATURES:cp=strpos(all_names[j],"+base");
                 if (cp < 0) break;
                 sprintf(helpstr,"%s",strmid(all_names[j],0,cp));
                 for (i=0;i<num_1D_states;i++)
                   {
                   fprintf(struct_def,
                       "sprintf(change_name[%d],\"%s\",\"%s%s\"); ",num_changes,"%s",helpstr,_1D_state_names[i]);
                    fprintf(struct_def,"change_ptr[%d]=(real*) %s%s; ",num_changes,helpstr,_1D_state_names[i]);
                    fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,type);
                    num_changes++;

                   }
                 break;

/* 2D Faelle  */
          case _2D_STATE:
          case _2D_GLOBAL: break;

          case _2D_INT_PARAM:fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s\"); ",num_changes,"%s",all_names[j]);
                 fprintf(struct_def,"change_ptr[%d]=(real*) &%s[1][1]; ",num_changes,all_names[j]);
                 fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,type);
                 num_changes++;
                 break;

          case _2D_REAL_PARAM:fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s\"); ",num_changes,"%s",all_names[j]);
                 fprintf(struct_def,"change_ptr[%d]=&%s[1][1]; ",num_changes,all_names[j]);
                 fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,type);
                 num_changes++;
                 break;
    
          case _2D_DERIVES:cp=strpos(all_names[j],"+base");
                 if (cp < 0) break;
                 sprintf(helpstr,"%s",strmid(all_names[j],0,cp));
                 for (i=0;i<num_2D_states;i++)
                   {
                   fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s%s\"); ",num_changes,"%s",helpstr,_2D_state_names[i]);
                    fprintf(struct_def,"change_ptr[%d]= %s%s; ",num_changes,helpstr,_2D_state_names[i]);
                    fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,type);
                    num_changes++;
                   }
                 break;

          case _2D_FEATURES:cp=strpos(all_names[j],"+base");
                 if (cp < 0) break;
                 sprintf(helpstr,"%s",strmid(all_names[j],0,cp));
                 for (i=0;i<num_2D_states;i++)
                   {
                   fprintf(struct_def,
                       "sprintf(change_name[%d],\"%s\",\"%s%s\"); ",num_changes,"%s",helpstr,_2D_state_names[i]);
                    fprintf(struct_def,"change_ptr[%d]=(real*) %s%s; ",num_changes,helpstr,_2D_state_names[i]);
                    fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,type);
                    num_changes++;

                   }
                 break;

          default:break;
          }
        }
   fprintf(struct_def,"printf(\"Searching for changes of main model parameters...\\n\");\n");

   fprintf(struct_def,"system(\"rm -f %s\");\n",PARAMCFILE);

/* cemos.par auswerten und die sensitivity.def auswerten */
/*new mycommand*/
        fprintf(struct_def,"sprintf(command,\"touch %s;\",sensitivity);\n","%s");
        fprintf(struct_def,"system(command);\n");
        fprintf(struct_def,"sprintf(command,\"cat %s %s > %s;\",sensitivity);\n","cemos.par","%s",PARAMCFILE);
        fprintf(struct_def,"system(command);\n");
/*end mycommand*/

/*  fprintf(struct_def,"system(\"touch %s\");\n",sensitivity);
  fprintf(struct_def,"system(\"cat cemos.par  %s > %s\");\n",sensitivity,PARAMCFILE);*/
/*   fprintf(struct_def,"system(\"cp cemos.par %s\");\n",PARAMCFILE);*/

   fprintf(struct_def,"if (system(\"gcc -E -traditional -I%s %s > %s\")!=0) std_error(\"Error in preprocessing the file cemos.par. Check for unresolvable include statements.\");\n",include_path,PARAMCFILE,PARAMTMPFILE);
/* sensitivity.def für das Übersteuern von Integrations- und Simulationsparametern auswerten */
   fprintf(struct_def,"system(\"rm -f %s\");\n",PARAMCFILE);
   fprintf(struct_def,"system(\"mv -f %s %s\");\n",PARAMTMPFILE,PARAMCFILE);
   fprintf(struct_def,"cpp2c(\"%s\",\"%s\");\n",PARAMCFILE,PARAMTMPFILE);
   fprintf(struct_def,"system(\"rm -f %s\");\n",PARAMCFILE);
  
   fprintf(struct_def,"sprintf(command,\"cat %s > %s;\",sensitivity);\n","%s",PARAMCFILE);
   fprintf(struct_def,"system(command);\n");

   fprintf(struct_def,"if (system(\"gcc -E -traditional -I%s %s > %s\")!=0)\n",include_path,PARAMCFILE,INTEGPARFILE);
   fprintf(struct_def,"{\n");
   fprintf(struct_def,"  sprintf(command,\"Error in preprocessing the file %s. Check for unresolvable include statements.\",sensitivity);\n","%s");
   fprintf(struct_def,"  std_error(command);\n");
   fprintf(struct_def,"}\n");
   fprintf(struct_def,"system(\"mv -f %s %s\");\n",INTEGPARFILE,PARAMCFILE);
   fprintf(struct_def,"cpp2c(\"%s\",\"%s\");\n",PARAMCFILE,INTEGPARFILE);

   /* Hier werden num_changes negativ uebergeben,
   um die komplette Namenspruefung fuer cemos.par zu unterdruecken */
   fprintf(struct_def,         "found_change=find_changes(\"%s\",change_name,change_ptr,change_type,%d);\n",PARAMTMPFILE,num_changes);
   fprintf(struct_def,"if (found_change==0) printf(\"No changes found.\\n\");");
   fprintf(struct_def,"fflush(stdout); \n");
   fprintf(struct_def,"system(\"rm -f %s\");\n",PARAMCFILE);
/*   fprintf(struct_def,"system(\"rm -f %s\");\n",PARAMTMPFILE);*/
   fprintf(struct_def,"return; \n");
   fprintf(struct_def,"} \n");
/* Routine zum Loeschen der abgeleiteten Variablen erzeugen */
          fprintf(res_derivs,"#include \"%s\" \n","struct.h");
          fprintf(res_derivs,"void reset_derivs(void) {\n");
          fprintf(res_derivs," long int number __attribute__((unused)) =0;\n");


/* 0D Faelle */
          fprintf(res_derivs," number=((long int) num_0D_states*sizeof(real));\n");
          fprintf(res_derivs," if (number>0)\n {\n");
          fprintf(res_derivs," memset(&SKKK[1],0,(size_t) number);\n");
          for (i=0;i<num_0D_states;i++)
                fprintf(res_derivs," S%s=0.0;\n",_0D_state_names[i]);

          for (j=1;j<num_0D_float_derivs;j++)
            {
            cp=strpos(_0D_float_derivs[j],"+");
            sprintf(helpstr,"%s",strmid(_0D_float_derivs[j],0,cp));
            if ( strpos(_0D_float_derivs[j],"variable") > 0)
             {
            /* eu Variablen ausschliessen CK 21/11/01 */
             if (strpos(helpstr,"eu") < 0)
              {
                fprintf(res_derivs," memset(&%sKKK[1],0,(size_t) number);\n",helpstr);
                for (i=0;i<num_0D_states;i++)
                      fprintf(res_derivs," %s%s=0.0;\n",helpstr,_0D_state_names[i]);
              }
             }
            }

            fprintf(res_derivs," }\n");

/* 1D Faelle */
          fprintf(res_derivs," number=((long int) num_1D_states*n_comp*sizeof(real));\n");
          fprintf(res_derivs," if (number>0)\n {\n");
          fprintf(res_derivs," memset(&SCCC[1][1],0,(size_t) number);\n");
          for (j=1;j<num_1D_float_derivs;j++)
            {
            cp=strpos(_1D_float_derivs[j],"+");
            sprintf(helpstr,"%s",strmid(_1D_float_derivs[j],0,cp));
            if ( strpos(_1D_float_derivs[j],"variable") > 0)
             {
            /* eu Variablen ausschliessen CK 21/11/01 */
             if (strpos(helpstr,"eu") < 0)
              {
                fprintf(res_derivs,
                   " memset(&%sCCC[1][1],0,(size_t) number);\n",helpstr);
              }
             }
            }
	  for (j=0;j<num_1D_int_derivs;j++)
            {
            cp=strpos(_1D_int_derivs[j],"+");
            sprintf(helpstr,"%s",strmid(_1D_int_derivs[j],0,cp));
            if ( strpos(_1D_int_derivs[j],"variable") > 0)
             {
             fprintf(res_derivs," memset(&%sCCC[1][1],0,(size_t) number);\n",helpstr);
             }
            }
            fprintf(res_derivs," }\n");

/* 2D - Faelle */
          fprintf(res_derivs," number=((long int) g_comp1*g_comp2*sizeof(real));\n");
          fprintf(res_derivs," if (num_2D_states>0)\n {\n");
          for (k=0;k<num_2D_states;k++)
           {
             fprintf(res_derivs," memset(&S%s[1][1],0,(size_t) number);\n",_2D_state_names[k]);
           
            for (j=1;j<num_2D_float_derivs;j++)
            {
            cp=strpos(_2D_float_derivs[j],"+");
            sprintf(helpstr,"%s",strmid(_2D_float_derivs[j],0,cp));
            if ( strpos(_2D_float_derivs[j],"variable") > 0)
             {
            /* eu Variablen ausschliessen CK 21/11/01 */
             if (strpos(helpstr,"eu") < 0)
              {
                fprintf(res_derivs," memset(&%s%s[1][1],0,(size_t) number);\n",helpstr,_2D_state_names[k]);
              }
             }
            }
	    for (j=0;j<num_2D_int_derivs;j++)
            {
            cp=strpos(_2D_int_derivs[j],"+");
            sprintf(helpstr,"%s",strmid(_2D_int_derivs[j],0,cp));
            if ( strpos(_2D_int_derivs[j],"variable") > 0)
             {
             fprintf(res_derivs," memset(&%s%s[1][1],0,(size_t) number);\n",helpstr,_2D_state_names[k]);
             }
            }
          }
          fprintf(res_derivs," }\n");
          fprintf(res_derivs,"}\n");

/* Routine zum Loeschen der abgeleiteten globalen Variablen erzeugen */
          fprintf(res_derivs,"void reset_glob_derivs(void) {\n");
          fprintf(res_derivs,"long int number __attribute__((unused)) =0;\n");
/* 1D - Faelle */
          fprintf(res_derivs,"if (num_1D_glob_derivs>0) \n{\n");
	  fprintf(res_derivs," number=((long int) num_1D_states*n_comp*sizeof(real));\n");
          for (j=0;j<num_1D_glob_derivs;j++)
            {
            cp=strpos(_1D_glob_derivs[j],"+");
            sprintf(helpstr,"%s",strmid(_1D_glob_derivs[j],0,cp));
            if ( strpos(_1D_glob_derivs[j],"variable") > 0)
             {
                fprintf(res_derivs," memset(&%sCCC[1][1],0,(size_t) number);\n",helpstr);
             }
            }
        fprintf(res_derivs,"}\n");
/* 2D - Faelle */

          fprintf(res_derivs,"if (num_2D_glob_derivs>0) \n{\n");
	  fprintf(res_derivs," number=((long int) num_2D_states*g_comp1*g_comp2*sizeof(real));\n");
          for (j=0;j<num_2D_glob_derivs;j++)
            {
            cp=strpos(_2D_glob_derivs[j],"+");
            sprintf(helpstr,"%s",strmid(_2D_glob_derivs[j],0,cp));
            if ( strpos(_2D_glob_derivs[j],"variable") > 0)
             {
                fprintf(res_derivs," memset(&%sDDD[1][1][1],0,(size_t) number);\n",helpstr);
             }
            }
        fprintf(res_derivs,"}\n");
        fprintf(res_derivs,"}\n");

/* Routine zum Vektorisieren der skalaren Variablen erzeugen */
          fprintf(res_derivs,"void make_global_0D_states(void) {\n");
          fprintf(res_derivs," if (num_0D_states>0)\n {\n");
          for (i=1;i<=num_0D_states;i++)
                fprintf(res_derivs," KKK[%d]=%s;\n",i,_0D_state_names[i-1]);
          fprintf(res_derivs," }\n");
          fprintf(res_derivs," if (num_0D_globals>0)\n {\n");
          for (i=1;i<=num_0D_globals;i++)
                fprintf(res_derivs," _0D_globals[%d]=%s;\n",i,_0D_global_names[i-1]);
          fprintf(res_derivs," }\n");
          fprintf(res_derivs,"}\n");


          fprintf(res_derivs,"void make_global_0D_rates(void) {\n");
          fprintf(res_derivs," if (num_0D_states>0)\n {\n");
          for (i=1;i<=num_0D_states;i++)
                fprintf(res_derivs," SKKK[%d]=S%s;\n",i,_0D_state_names[i-1]);
          fprintf(res_derivs," }\n");
          fprintf(res_derivs,"}\n");

/* ... und wieder rueckwaerts */
          fprintf(res_derivs,"void make_local_0D_states(void) {\n");
          fprintf(res_derivs," if (num_0D_states>0)\n {\n");
          for (i=1;i<=num_0D_states;i++)
                fprintf(res_derivs," %s=KKK[%d];\n",_0D_state_names[i-1],i);
          fprintf(res_derivs,"}\n");
          fprintf(res_derivs,"}\n");

          fprintf(res_derivs,"void make_local_0D_rates(void) {\n");
          fprintf(res_derivs," if (num_0D_states>0)\n {\n");
          for (i=1;i<=num_0D_states;i++)
                fprintf(res_derivs,"S%s=SKKK[%d];\n",_0D_state_names[i-1],i);
          fprintf(res_derivs," }\n");
          fprintf(res_derivs,"}\n");

/* Routine zur Ermittlung von Pointern auf abgeleitete 0D Variablen erzeugen */
          fprintf(struct_def,"real* get_0D_store_ptr(name search) {\n");
          fprintf(struct_def,"real *ptr=NULL;\n");
          fprintf(struct_def,"int i;\n");
          fprintf(struct_def,"for (i=1;i<=num_0D_states;i++) {\n");
          fprintf(struct_def,"  if (strpos(search,_0D_state_names[i]) > 0 ) \n");
          fprintf(struct_def,"   { \n");
          for (j=0;j<num_0D_float_derivs;j++)
            {
            cp=strpos(_0D_float_derivs[j],"+");
            sprintf(helpstr,"%s",strmid(_0D_float_derivs[j],0,cp));
            if ( strpos(_1D_float_derivs[j],"variable") > 0)
             {
             fprintf(struct_def,"    if (strpos(search,\"%s\") >= 0 && strlen(search)==(strlen(_0D_state_names[i])+strlen(\"%s\"))) ptr=&%sKKK[i];\n",helpstr,helpstr,helpstr);
             }
            }
          fprintf(struct_def,"   }\n  }\n");
          fprintf(struct_def,"return(ptr);}\n");



/* Routine zur Ermittlung von Pointern auf abgeleitete 1D Variablen erzeugen */
          fprintf(struct_def,"real* get_1D_store_ptr(int box, name search) {\n");
          fprintf(struct_def,"real *ptr=NULL;\n");
          fprintf(struct_def,"int i;\n");
          fprintf(struct_def,"for (i=1;i<=num_1D_states;i++) {\n");
          fprintf(struct_def,"  if (strpos(search,_1D_state_names[i]) > 0 ) \n");
          fprintf(struct_def,"   { \n");
          for (j=0;j<num_1D_float_derivs;j++)
            {
            cp=strpos(_1D_float_derivs[j],"+");
            sprintf(helpstr,"%s",strmid(_1D_float_derivs[j],0,cp));
            if ( strpos(_1D_float_derivs[j],"variable") > 0)
             {
             fprintf(struct_def,"    if (strpos(search,\"%s\") >= 0 && strlen(search)==(strlen(_1D_state_names[i])+strlen(\"%s\"))) ptr=&%sCCC[i][box];\n",helpstr,helpstr,helpstr);
             }
            }
          for (j=0;j<num_1D_glob_derivs;j++)
            {
            cp=strpos(_1D_glob_derivs[j],"+");
            sprintf(helpstr,"%s",strmid(_1D_glob_derivs[j],0,cp));
            if ( strpos(_1D_glob_derivs[j],"variable") > 0)
             {
             fprintf(struct_def,"    if (strpos(search,\"%s\") >= 0 && strlen(search)==(strlen(_1D_state_names[i])+strlen(\"%s\"))) ptr=&%sCCC[i][box];\n",helpstr,helpstr,helpstr);
             }
            }
	  fprintf(struct_def,"   }\n  }\n");
          fprintf(struct_def,"return(ptr);}\n");
         }
    stop_label:;
    } /* ende von for (pass=0 ...  */
  free(helpstr);
  free(input);
  free(_1D_int_derivs);
  free(_1D_float_derivs);
  free(_1D_global_names);
  free(_1D_glob_derivs);
  free(_1D_state_names);
  free(all_types);
  free(all_names);

  fclose(res_derivs);
  fclose(struct_h);
  fclose(struct_def);
  return(0);
}



#include "pass0_p.c"
#include "pass1_p.c"
#include "build_structures.c"

