/***********************************************************************
**                                                                    **
** CEMoS Version 4.1.2 Copyright(C) 2023 Frank Hamberg, Cora Kohlmeier**
**                                                                    **
***********************************************************************/
#include <math.h>
#include "struct.h"
#include "constructs.h"

#define MAXSTPE 1000
/* #define TINY 1.0e-30 */
#define TINY 1.0e-12

extern real *ytemp,*y,*dydx,*oldrate;
extern void control_main(int,real, real,real*, real*);
void shrunk(real,real,long,real,real,char*);

void euler(real x1,real x2,real *h)
{
long nstp;
long shr_delt;
long i;
real x,hnext,hrel,hnul=0.,abbr;
string error;


x=x1;

make_global_0D_states();
make_local(DDD,CCC,KKK,y);
		
for (nstp=1;nstp<=MAXSTPE;nstp++) 
 {

 if (((x+*h)-x2) > 0.0) *h=x2-x;

 control_main(0,x,*h,y,dydx);

 for (i=1;i<=nvar;i++) ytemp[i]=*h*dydx[i];

 shr_delt=0;

 /* Ueberpruefung auf relrate */
 hrel=*h;
 for (i=1;i<=nvar;i++)
  {
  if (oldrate[i]*dydx[i]<0.) abbr=relchange; else abbr=relrate;

  if (y[i]>0. || y[i]<0.)
   {
   if (fabs(ytemp[i]/y[i]) >= abbr)
    { 
    hnext=abbr*fabs(y[i]/dydx[i]);
    if (hnext<hrel)
     {
     shr_delt=i;
     hrel=hnext;
     }
    }
   }
  }


  /* Ueberpruefung auf Null-Unterschreitung */
if (method > 0)
 {
 hnul=hrel;
 for (i=1;i<=nvar;i++)
  {
  if (y[i]+ytemp[i]<0.) 
   {
   hnext=-relchange*y[i]/dydx[i];
   if (hnext<hnul)
    {
    shr_delt=-i;
    hnul=hnext;
    }
   }
  }   
 }

 if (shr_delt<0)  hnext=hnul; else hnext=hrel;
 if (shr_delt<0)  i=-shr_delt; else i=shr_delt;  

 if (i!=0) shrunk(x,*h,i,y[i],ytemp[i],"euler");

 *h=hnext;
 x+=hnext;

 for (i=1;i<=nvar;i++) 
  {
  y[i]+=*h*dydx[i];
  oldrate[i]=dydx[i];
  }

 if (shr_delt==0) *h=x2-x;

 if ((x-x2)*(x2-x1) >= 0.0) 
  {
  *h=maxdelt;
  make_global(y,KKK,CCC,DDD);
  make_local_0D_states();

  for (i=1;i<=nvar;i++) oldrate[i]=0.;
  return;
  }
  if (hnext <= mindelt) 
  {
     sprintf(error,"Step size too small in Euler. delt=%g",(double) hnext);
     std_error(error);
   }

 }
std_error("Too many steps in routine Euler");
}
#undef MAXSTPE
#undef TINY
#undef NRANSI
