/***********************************************************************
**                                                                    
** CEMoS Version 4.1.2 Copyright(C) 2023 Frank Hamberg, Cora Kohlmeier**
**                                                                    
**                                                                    
** Changes:                                                                   
**   05/08/01: check for multiple storage of variables
**                                                                 
***********************************************************************/
#include "stdinc.h"
#include "prepare.h"
#include "struct.h"
extern real* get_0D_store_ptr(char*);
void get_0D_stores(char *filename,int pre_dim,unsigned int *num_0D_stores,real ***_0D_store_ptr,name **_0D_stores)
{

   FILE *para_file;
   int cp,i,k;
   int found=-1,counter=0;
   unsigned int i1,k1;
   char *input,*list,*helpstr,*compstr;
   name *pre_stores,helpname;
   name *nsp;
   real **pre_store_ptr;
   real *pointer=(real*) NULL,**ffp;

   para_file=fopen(filename,"rb");

   input=(char *) malloc(_BUFSIZE_*sizeof(char));
   helpstr=(char *) malloc(_BUFSIZE_*sizeof(char));
   compstr=(char *) malloc(_BUFSIZE_*sizeof(char));
   list=(char *) malloc(_BUFSIZE_*sizeof(char));

   pre_stores=(name *) malloc(pre_dim*sizeof(name));
   pre_store_ptr=(real **) malloc(pre_dim*sizeof(real*));
   if (!pre_stores) std_error("Internal error: allocation failure for pre_stores");
   if (!pre_store_ptr) std_error("Internal error: allocation failure for pre_store_ptr");

/* store-Block suchen */
   while (found < 0)
    {
    if (fgets(helpstr,100,para_file) == 0) goto stop_label;
    found=strpos(helpstr,_store_0D_);
    }

#if defined (__verbose__)
    printf("Store variables:\n");
    fflush(stdout);
#endif
    while ( fgets(input,100,para_file) != 0 )
      {
      if (strpos(input,"#")>=0) sprintf(input,"%s","");

      while(strpos(input," ")==0)
           sprintf(input,"%s",strmid(input,1,(int) strlen(input)-1));

      if (strpos(input,change_id) ==0) break;
      if (strpos(input,_store_)==0) break;
      if (strpos(input,_store_2D_)==0) break;

      if (strpos(input,_keyword_) >=0) sprintf(input,"%s","");

      if (strlen(input)>0)
       {
#if defined (__verbose__)
        printf("%s",input);
        fflush(stdout);
#endif
        strtrim(input);
        sprintf(list,"%s",input);
        while(strlen(list)>0)
         {
         cp=strpos(list,";");
         if (cp>0)
          {
          sscanf(strmid(list,0,cp),"%s",input);
          sprintf(list,"%s",strmid(list,cp+1,(int) strlen(list)-cp-1));
          }
          else
          {
          sscanf(list,"%s",input);
          sprintf(list,"%s","");
          }
          found=0;
          sprintf(helpstr,"%s",input);
          i=(int) strlen(helpstr);
          for (k=1;k<=num_0D_states;k++)
           {
           sprintf(compstr,"%s",_0D_state_names[k]);
           if (strpos(compstr,helpstr) == 0 && (int)strlen(compstr)==i)
            {
            pointer=(real*) &KKK[k];
            found=1;
            break;
            }
           }
           if (found==0)
            for (k=1;k<=num_0D_globals;k++)
            {
            sprintf(compstr,"%s",_0D_global_names[k]);
            if (strpos(compstr,helpstr) == 0 && (int)strlen(compstr)==i)
             {
             pointer=(real*) &_0D_globals[k];
             found=1;
             break;
             }
            }
           if (found==0)
            for (k=1;k<=num_0D_float_derivs;k++)
            {
            sprintf(compstr,"%s",_0D_float_derivs[k]);
            if (strpos(compstr,helpstr) == 0 && (int)strlen(compstr)==i)
              {
              sprintf(helpname,"%s",strmid(helpstr,0,__namelength));
              pointer=get_0D_store_ptr(helpname);
              found=1;
              break;
              }
             }

	   if (pointer==(real*) 0 || found==0)
            {
            printf("Error in 0D-store command: No such Variable %s !!\n",helpstr);
            std_error("Check the the store-commands !!");
            }
            sprintf(pre_stores[*num_0D_stores],"%s(0)",compstr);
	    pre_store_ptr[*num_0D_stores]=pointer;

            (*num_0D_stores)++;

       }
      }
    }
    *_0D_store_ptr=(real **) malloc(*num_0D_stores*sizeof(real *));
    *_0D_stores=(name *) malloc(*num_0D_stores*sizeof(name));
    ffp=*_0D_store_ptr;
    nsp=*_0D_stores;
/*  endgueltige Liste erstellen */
/* doppelte Eintraege aussortieren */

    counter = 0;
    for (i1=0;i1<*num_0D_stores;i1++)
    {
      found = 0;
      for (k1=0;k1<i1;k1++)
      {
         if ( i1 != k1 && pre_store_ptr[i1] == pre_store_ptr[k1]) 
         {
#if defined(__verbose__)
         printf("Warning: Removed multiple stored variable %s\n",pre_stores[i1]);
#endif
         found = 1;
         }
      }
      if (found == 0) 
      {
      sprintf(nsp[counter],"%s",pre_stores[i1]);
      ffp[counter]=pre_store_ptr[i1];
      counter++;   
      }
    }
    *num_0D_stores=counter;

    stop_label:
    free(input);
    free(helpstr);
    free(compstr);
    free(list);
    free(pre_stores);
    free(pre_store_ptr);
    fclose(para_file);
}
