/***********************************************************************
**                                                                    
** CEMoS Version 4.1.2 Copyright(C) 2023 Frank Hamberg, Cora Kohlmeier**
**                                                                    
**                                                                    
** Changes:                                                                   
**   05/08/01:    check for multiple storage of variables
**   2023/02/15:  check for valid index range of variables
**                                                                 
***********************************************************************/
#include "stdinc.h"
#include "prepare.h"
#include "struct.h"
extern real* get_1D_store_ptr(int,char*);
void get_1D_stores(char *filename,int pre_dim,int max_len,unsigned int *num_1D_stores,real ***_1D_store_ptr,name **_1D_stores)
{

   FILE *para_file;
   int ip1,ip2,cp,i,k;
   int found=-1,counter=0,check=0,*flags;
   unsigned int i1,k1;
   char *input,*list,*helpstr,*compstr;
   name *pre_stores,helpname;
   name *nsp;
   real **pre_store_ptr;
   real *pointer=(real*) NULL,**ffp;

   para_file=fopen(filename,"rb");

   input=(char *) malloc(_BUFSIZE_*sizeof(char));
   helpstr=(char *) malloc(_BUFSIZE_*sizeof(char));
   compstr=(char *) malloc(_BUFSIZE_*sizeof(char));
   list=(char *) malloc(_BUFSIZE_*sizeof(char));
   flags=ivector(1,_BUFSIZE_);
   for (i=1;i<=_BUFSIZE_;i++) flags[i]=0;

   pre_stores=(name *) malloc(pre_dim*max_len*sizeof(name));
   pre_store_ptr=(real **) malloc(pre_dim*max_len*sizeof(real*));
   if (!pre_stores) std_error("Internal error: allocation failure for pre_stores");
   if (!pre_store_ptr) std_error("Internal error: allocation failure for pre_store_ptr");

/* store-Block suchen */
   while (found < 0)
    {
    if (fgets(helpstr,100,para_file) == 0) goto stop_label;
    found=strpos(helpstr,_store_);
    if (strpos(helpstr,_store_0D_)==0) found=-1;
    }

#if defined (__verbose__)
    printf("Store variables:\n");
    fflush(stdout);
#endif
    while ( fgets(input,100,para_file) != 0 )
      {
      if (strpos(input,"#")>=0) sprintf(input,"%s","");

      while(strpos(input," ")==0)
           sprintf(input,"%s",strmid(input,1,(int) strlen(input)-1));

      if (strpos(input,change_id) ==0) break;
      if (strpos(input,_store_0D_)==0) break;
      if (strpos(input,_store_2D_)==0) break;

      if (strpos(input,_keyword_) >=0) sprintf(input,"%s","");

      if (strlen(input)>3)
       {
#if defined (__verbose__)
        printf("%s",input);
        fflush(stdout);
#endif
        strtrim(input);
        sprintf(list,"%s",input);
        while(strlen(list)>0)
         {

         cp=strpos(list,";");
         if (cp>0)
          {
          sscanf(strmid(list,0,cp),"%s",input);
          sprintf(list,"%s",strmid(list,cp+1,(int) strlen(list)-cp-1));
          }
          else
          {
          sscanf(list,"%s",input);
          sprintf(list,"%s","");
          }
          ip1=strpos(input,"[");
          ip2=strpos(input,"]");
          if (ip2>ip1)
           {
            sprintf(helpstr,"%s",strmid(input,0,ip1));
           }
           else
           {
            sprintf(helpstr,"---------");
           }

          found=0;
          i=(int) strlen(helpstr);
          for (k=1;k<=num_1D_states;k++)
           {
           sprintf(compstr,"%s",_1D_state_names[k]);
           if (strpos(compstr,helpstr) == 0 && (int)strlen(compstr)==i)
            {
            pointer=(real*) &CCC[k][0];
            found=1;
            break;
            }
           }
           if (found==0)
            for (k=1;k<=num_1D_globals;k++)
            {
            sprintf(compstr,"%s",_1D_global_names[k]);
            if (strpos(compstr,helpstr) == 0 && (int)strlen(compstr)==i)
             {
             pointer=(real*) &_1D_globals[k][0];
             found=1;
             break;
             }
            }
           if (found==0)
            for (k=1;k<=num_1D_float_derivs;k++)
            {
            sprintf(compstr,"%s",_1D_float_derivs[k]);
            if (strpos(compstr,helpstr) == 0 && (int)strlen(compstr)==i)
              {
              sprintf(helpname,"%s",strmid(helpstr,0,__namelength));
              pointer=get_1D_store_ptr(0,helpname);
              found=1;
              break;
              }
             }
           if (found==0)
            for (k=1;k<=num_1D_glob_derivs;k++)
            {
            sprintf(compstr,"%s",_1D_glob_derivs[k]);
            if (strpos(compstr,helpstr) == 0 && (int)strlen(compstr)==i)
              {
              sprintf(helpname,"%s",strmid(helpstr,0,__namelength));
              pointer=get_1D_store_ptr(0,helpname);
              found=1;
              break;
              }
             }

	   if (pointer==(real*) 0 || found==0)
            {
            printf("Error in 1D-store command: No such Variable %s !!\n",helpstr);
            std_error("Check the the store-commands !!");
            }

          if (set_flags(strmid(input,ip1+1,ip2-ip1-1),flags) > max_len)
          {
            printf("Error in 1D-store command: Wrong index in statement %s !!\n",input);
            std_error("Check the the store-commands !!");
          }  
  
          for (k=1;k<=max_len;k++)
            {
              if (flags[k] != 0)
               {
               sprintf(pre_stores[*num_1D_stores],"%s(%d)",helpstr,k);
	           pre_store_ptr[*num_1D_stores]=&pointer[k];
               flags[k]=0;
               check=1;
               (*num_1D_stores)++;
               }
             }
	         if (check==0)
	         {
	           printf("Error in 1D-store command: Exceeding index in statement %s !!\n",input);
	           std_error("Check the the store-commands !!");
	         }  
	         check=0;
          }
       }
      }
    *_1D_store_ptr=(real **) malloc(*num_1D_stores*sizeof(real *));
    *_1D_stores=(name *) malloc(*num_1D_stores*sizeof(name));
    ffp=*_1D_store_ptr;
    nsp=*_1D_stores;
/*  endgueltige Liste erstellen */
/* doppelte Eintraege aussortieren */

    counter = 0;
    for (i1=0;i1<*num_1D_stores;i1++)
    {
      found = 0;
      for (k1=0;k1<i1;k1++)
      {
         if ( i1 != k1 && pre_store_ptr[i1] == pre_store_ptr[k1]) 
         {
#if defined(__verbose__)
         printf("Warning: Removed multiple stored variable %s\n",pre_stores[i1]);
#endif
         found = 1;
         }
      }
      if (found == 0) 
      {
      sprintf(nsp[counter],"%s",pre_stores[i1]);
      ffp[counter]=pre_store_ptr[i1];
      counter++;   
      }
    }
    *num_1D_stores=counter;
    
    stop_label:
    free(input);
    free(helpstr);
    free(compstr);
    free(list);
    free_ivector(flags,1,max_len);
    free(pre_stores);
    free(pre_store_ptr);
    fclose(para_file);
}
