/************************************************************************
**
** CEMoS Version 4.1.2 Copyright(C) 2023 Frank Hamberg, Cora Kohlmeier**
** last changes:
** 2012/07/28 fh: endtime, starttime, storetime zusätzlich eingebaut für Verwendung in cemos.par und im Modell                                            **
***********************************************************************/
#include "prepare.h"
#include "struct.h"
#include "main.h"
#include "methods.h"
#include <time.h>

#include <signal.h>
#include <unistd.h>
#include <sys/types.h>

#if defined (__with_slave__)
#include <sys/ipc.h>
#include <sys/shm.h>
#endif
#define _GNU_SOURCE
#include <getopt.h>

extern void odeint(real,real,real*);
extern void euler(real,real,real*);
extern void back_euler(real,real,real*);
extern void fixstep(real,real);
extern void no_integration(real,real);
extern void control_main(int,real, real,real*, real*);
extern void write_info(char*);
extern void get_0D_stores(char*,int,unsigned int*,real***,name**);
extern void get_1D_stores(char*,int,int,unsigned int*,real***,name**);
extern void get_2D_stores(char *,int,unsigned int*,real****,name**);
extern void struct_def(string);
extern void read__sub(string);
extern void model(void);
#if defined (__ljapunov__)
  extern void ljapunov(void);
#endif

/* global arrays for integration */
real *yscal,*y,*dydx,*dysav,*ysav,*ytemp,*yerr,*dym,*dyt,*yt,*oldrate;
real *ak2,*ak3,*ak4,*ak5,*ak6;

int main(int argc, char *argv[])
{

/*
Variablen und Pointer fuer die Kommunikation ueber shared memory
*/
#if defined (__with_slave__)
  void master_reads_slave(int);
  void slave_sets_master(int);

  void *shared_memory = (void *) 0;
  key_t secret_key;
  int shmid;
  int interface_box = -1;
  int kill(pid_t pid, int sig);

  int i_am_slave=0,i_am_master=0;
  string slave_dir,slave_command,slave_info,slave_test;

#endif

/* Header des Result-Files (ERSEM)*/
  struct header
   {
      unsigned int n_stor;
      float startim;
      float endtim;
      float maxdelt;
      float outdelt;
      float year;
      float cycle;
      char sdir[__namelength];
   } oh;

/* Variablen fuer die netCDF-Ausgabe */
/* generell */
   int ncid=0;			/* netCDF id */
   long netcdf_store=0;
   int status;
   int time_id=0;           /* Identifier fuer die Zeitinformation */
   int timedim;

   /* fuer die CEMoS Header-Informationen in netCDF */
   int  numstor_id=0;
   int  numstor_2D_id=0;
   int  starttime_id=0;
   int  endtime_id=0;
   int  maxdelt_id=0;
   int  outdelt_id=0;
   int  year_id=0;
   int  cycle_id=0;
   int directory_id=0;
   int netcdf_output=0;
   int proceed_id=0;
   float value=0.0;

/* 1D */
   int dims[1];  
   int *var_id=NULL;            /* netCDF-Identifier fuer die Speichervariablen */
   long netcdf_start[]={0};
   long netcdf_stop[]={1};

/* 2D */
   int dims_2D[3];
   int dim1_2D;
   int dim2_2D;
                  
   int *_2D_var_id=NULL;            /* netCDF-Identifier fuer die Speichervariablen */
   size_t netcdf_start_2D[3];
   size_t netcdf_count_2D[3];

/*================================*/

  FILE *outfile;
  FILE *parfile;
  FILE *pidfile;

  time_t tstart,start,end;

  string param_file,output_file,sensitivity,result_path;
  string report;
  string __lock,__unlock;
/*storestart gibt an, ob vor oder nach der Integration gespeichert wird
default ist 0 = Speichern vor der Integration*/

  int num_ints=1,num_ints_cemospar=1,storestart=0,overrule_on=0;
  int slave_control __attribute__((unused)) =-1;
  int i,k,l;
  int total_0D_indexed, total_indexed, total_2D_indexed;
  int *m_method,*m_assign,*flags;
  long j;
  unsigned int num_stores,ui;
  unsigned int  num_0D_stores, num_2D_stores;
  unsigned long steps,step,next_store,store_step;
  unsigned long *next_int,*step_len;
  string helpstr;
  name *stores=NULL;
  name *stores_0D=NULL;
  name *store_0D_names __attribute__ ((unused));
  name *store_names __attribute__ ((unused));
  name *stores_2D __attribute__ ((unused));

  name main_dir;
  name helpname;
  string info_file,info_filename,overrule,param_name,resulttype,slave_key;
  real *m_acc,*m_relchange,*m_relrate,*m_maxdelt,*m_mindelt;
  real startk,end_of_int,calc_delt,mmdelt;
  real *start_of_int,*delt_of_int;

  float *store_0D_vector __attribute__((unused))=NULL;
  float *store_vector __attribute__((unused))=NULL;
  float *store_2D_vector __attribute__((unused))=NULL;

  real **store_ptr=NULL;
  real **store_0D_ptr=NULL;
  real **store_1D_ptr=NULL;
  real ***store_2D_ptr=NULL;

  real info_time=0.0,info_step=10.,start_store;
  real infofiletime=-1.0,infofilestep=0.0;

  outfile = (FILE*) NULL;
/*  int digit_optind = 0;
  int this_option_optind = optind ? optind : 1;*/
  int option_index = 0;
  int option;
  static struct option long_options[] =
   {
    {"overrule", 1, 0, 0},
    {"outputfile", 1, 0, 0},
    {"resulttype", 1, 0, 0},
    {"slave_key", 1, 0, 0},
    {"result_path", 1, 0, 0},
    {"sensitivity", 1, 0, 0},
    {0, 0, 0, 0}
   };
/* Default result-type */
  sprintf(resulttype,"outc");
  netcdf_output=0;
/* Default result-filename */
   sprintf(output_file,"result");
   sprintf(info_file,"%s","result");
   sprintf(resultname,"%s",output_file);
   sprintf(sensitivity,".sensitivity.def");

 /* remember start time */
  time(&tstart);

/* PID-file anlegen */
   pidfile =fopen("cemos.pid","wb");
   fprintf(pidfile,"%d",(int) getpid());
   fclose(pidfile);

/* Evaluation of start parameters   */
  
  while (1)
    {
      option = getopt_long_only(argc, argv, "o:f:r:k:p:s:",long_options, &option_index);
      if (option == -1) break;

      switch (option)
        {
        case 0:
              switch (option_index)
	      {
		case 0:
		      sprintf(overrule,"%s",optarg);
		      overrule_on=1;
		      break;
		case 1:
		      sprintf(output_file,"%s",optarg);
		      sprintf(info_file,"%s",optarg);
		      break; 
		case 2:
		      sprintf(resulttype,"%s",optarg);
		      break;
		case 3:
		      sprintf(slave_key,"%s",optarg);
		      slave_control=1;
		      break;
		case 4:
		      sprintf(result_path,"%s",optarg);
		      break;
		case 5:  
		      sprintf(sensitivity,"%s",optarg);
		      break;
	      } 
          break;

         case 'o':
	      sprintf(overrule,"%s",optarg);
	      overrule_on=1;
	      break;

        case 'f':
	      sprintf(output_file,"%s",optarg);
	      sprintf(info_file,"%s",optarg);
	      break; 

        case 'r':
	      sprintf(resulttype,"%s",optarg);
	      break;

        case 'k':
	      sprintf(slave_key,"%s",optarg);
	      slave_control=1;
	      break;

        case 'p':
	      sprintf(result_path,"%s",optarg);
	      break;

        case 's':
	      sprintf(sensitivity,"%s",optarg);
	      break;

        case '?':
          break;

        default:
          printf ("Usage: cemos-model [options]:\n");
          printf("--overrule [-o] filename \n");
          printf("--outputfile [-f] filename without extension\n");
          printf("--resulttype [-r] extension for output file [outc] or [nc] or [csv]\n");
          printf("--slave_key [-k] memory key for slave model\n");
          printf("--result_path [-p] target directory for results\n");
          printf("--sensitivity [-s] parameter file for sensitivity analyses\n");
          std_error("Wrong usage of cemos-model");
        }
    }

  if (optind < argc)
    {
      printf ("unreckognised arguments in ARGV: ");
      while (optind < argc)
      printf ("%s ", argv[optind++]);
      printf ("\n");
      std_error("Unknown arguments for cemos-model");
    }

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/    

#if defined (__with_slave__)
  if (slave_control==1)
   {
   sscanf(slave_key,"%d",&k);
   secret_key = (key_t) k;
   i_am_slave = 1;
   }
   else
   {
   secret_key =(key_t) 0;
   }
#endif
/* Feld zum Auswerten von Parameterlisten in Aenderungen */
  flags=ivector(1,n_comp);

/* Semaphore anlegen, damit nicht verschiedene Instanzen nicht zeitgleich die "sensitivity.par" zugreifen */
/* Das Kommando erzeugt die Datei "cemos.lock"; 
   wenn diese wegen vorherigen Abbruchs noch besteht wir nach 5 Sekunden (-l 5) der Lock aufgehoben und dann nach 2 Sekunden (-s 2) weitgemacht */
  sprintf(__lock,"%s","lockfile -l 2 -s 5 cemos.lock");
/* Der normale unlock erfolgt später (nach read_sub() ) mit diesem Kommando: */
  sprintf(__unlock,"%s","rm -f cemos.lock");

/* Dies wird nicht getan, wenn nicht explizit in .tkcemosset gesetzt (MULTI_INSTANCES 1), da in gewissen Konstellationen (VMWare shared folders) seltsame Effekte auftreten. */
#if defined (__MULTI_INSTANCES__)
  printf("Creating lock on dynamic parameter files\n");
  fflush(stdout);
  system(__lock);
#endif
  
/* Modellstrukturen erzeugen */

struct_def(sensitivity);

  sprintf(param_file,"%s",PARAMFILE);
  if ((parfile =fopen(param_file,"rb")) == (FILE *) NULL )
    {
    sprintf(report,"No File : %s \n",param_file);
    std_error(report);
    }
   fclose(parfile);

/* Unused boxes suchen und auswerten */
   get_list(param_file,unused_boxes,n_comp,flags);
   l_comp=0;
   for (i=1;i<=n_comp;i++)
       if (flags[i] == 0)
        {
        l_comp++;
        i_com[l_comp]=i;
/*        printf("icom[%d]=%d ",l_comp,i_com[l_comp]); */
        }

/* Vektorlaenge fuer die Integration */
   nvar=num_0D_states+num_1D_states*l_comp + num_2D_states*g_comp1*g_comp2;
   y=vector(1,nvar);
   dydx=vector(1,nvar);
   ytemp=vector(1,nvar);
   yerr=vector(1,nvar);

/* Integrationsparameter einlesen */
#if defined (__verbose__)
   printf("Reading simulation parameters from cemos.par\n");
   fflush(stdout);
#endif
   get_int(param_file,_simu_par_,"recalc_globals",&recalc_globals);
   get_int(param_file,_simu_par_,"liap",&liap);
   get_int(param_file,_simu_par_,"multi",&num_ints_cemospar);
   get_flt(param_file,_simu_par_,"starttime",&starttime);
   if (starttime<0.0) 
   {  
       get_flt(param_file,_simu_par_,"startim",&startim);
       starttime=startim;
   } 
   else
   {
     startim=starttime;
   }
   get_flt(param_file,_simu_par_,"endtime",&endtime);
   if (endtime<0.0) 
   {  
       get_flt(param_file,_simu_par_,"endtim",&endtim);
       endtime=endtim;
   } 
   else
   {
     endtim=endtime;
   }
   get_flt(param_file,_simu_par_,"storetime",&storetime);
   if (storetime<0.0) 
   {  
       get_flt(param_file,_simu_par_,"storetim",&storetim);
       storetime=storetim;
   } 
   else
   {
     storetim=storetime;
   }
   
   get_flt(param_file,_simu_par_,"outdelt",&outdelt);
   get_flt(param_file,_simu_par_,"infotim",&infofiletime);
   if (infofiletime<0.0) get_flt(param_file,_simu_par_,"infotime",&infofiletime);
   
   get_flt(param_file,_simu_par_,"year",&year);
   get_flt(param_file,_simu_par_,"cycle",&cycle);
   get_int(param_file,_simu_par_,"storestart",&storestart);
   
   num_ints=num_ints_cemospar;
/* Output Type holen/suchen */
   get_str(param_file,_simu_par_,"resulttype",resulttype);
   
   if (overrule_on == 1)
    {
#if defined (__verbose__)
    printf("Overruling simulation parameters from %s\n",overrule);
    fflush(stdout);
#endif
   get_int(overrule,_simu_par_,"recalc_globals",&recalc_globals);
   get_int(overrule,_simu_par_,"liap",&liap);
   get_int(overrule,_simu_par_,"multi",&num_ints);
   get_flt(overrule,_simu_par_,"startim",&startim);
   get_flt(overrule,_simu_par_,"endtim",&endtim);
   get_flt(overrule,_simu_par_,"outdelt",&outdelt);
   get_flt(overrule,_simu_par_,"storetim",&storetim);
   get_flt(overrule,_simu_par_,"infotim",&infofiletime);
   get_flt(overrule,_simu_par_,"year",&year);
   get_flt(overrule,_simu_par_,"cycle",&cycle);
   get_int(overrule,_simu_par_,"storestart",&storestart);

/* Output Type holen/suchen */
   get_str(overrule,_simu_par_,"resulttype",resulttype);
  }
   // zur Verwendung im Modell, wieder fuer Uebereinstimmung sorgen
   if (starttime>startim || starttime<startim) starttime=startim;
   if (endtime>endtim || endtime<endtim) endtime=endtim;
   if (storetime>storetim || storetime<storetim) storetime=storetim;
   calc_delt=endtim;

   if (startim>endtim || storetim>endtim)
      std_error("Starttime or Storetime is greater than endtime !!");

   m_method=ivector(1,num_ints);
   m_mindelt=vector(1,num_ints);
   m_maxdelt=vector(1,num_ints);
   m_acc=vector(1,num_ints);
   m_relrate=vector(1,num_ints);
   m_relchange=vector(1,num_ints);
   m_assign=ivector(1,num_ints);
   start_of_int=vector(1,num_ints);
   step_len=lvector(1,num_ints);
   next_int=lvector(1,num_ints);
   delt_of_int=vector(1,num_ints);

#if defined (__verbose__)
   printf("Reading integration parameters\n");
   fflush(stdout);
#endif
   for (k=1;k<=num_ints_cemospar;k++)
     {
     m_method[k]=-9999;
     m_maxdelt[k]=-9999.;
     m_mindelt[k]=-9999.;
     m_relrate[k]=-9999.;
     m_relchange[k]=-9999.;
     m_acc[k]=-9999.;

     if (num_ints_cemospar==1)
      {
      sprintf(param_name,"%s",_integ_par_);
      m_assign[1]=-1;
      }
      else
      {
      sprintf(helpname,"%d",k);
      sprintf(param_name,"%s%s",_integ_par_,helpname);
      m_assign[k]=k;
      }

      get_int(param_file,param_name,"method",&m_method[k] );
      if (m_method[k]==-9999) 
      {
          sprintf(helpstr,"No method defined in section %s with multi=%d",param_name,num_ints);
          std_error(helpstr);
      }
      get_flt(param_file,param_name,"mindelt",&m_mindelt[k]);
      if (m_mindelt[k]<=-9999.0) 
      {
          sprintf(helpstr,"No mindelt defined in section %s  with multi=%d",param_name,num_ints);
          std_error(helpstr);
      }
      get_flt(param_file,param_name,"maxdelt",&m_maxdelt[k]);
      if (m_maxdelt[k]<=-9999.0) 
      {
          sprintf(helpstr,"No maxdelt defined in section %s with multi=%d",param_name,num_ints);
          std_error(helpstr);
      }

       switch(abs(m_method[k]))
        {
        case _Fixstep:break;

        case _Euler:get_flt(param_file,param_name,"relrate",&m_relrate[k]);
                if (m_relrate[k]<=-9999.0) 
                {
                        sprintf(helpstr,"No relrate defined in section %s with multi=%d",param_name,num_ints);
                        std_error(helpstr);
                }
               get_flt(param_file,param_name,"relchange",&m_relchange[k]);
                if (m_relchange[k]<=-9999.0) 
                {
                        sprintf(helpstr,"No relchange defined in section %s with multi=%d",param_name,num_ints);
                        std_error(helpstr);
                }
               if (oldrate==(real*) NULL)
                {
                oldrate=vector(1,nvar);
                for (j=1;j<=nvar;j++) oldrate[j]=0.0;
                }
               break;
        case _RK23:
        case _RK45:
        case _RKCK:get_flt(param_file,param_name,"accuracy",&m_acc[k]);
                if (m_acc[k]<=-9999.0) 
                {
                        sprintf(helpstr,"No accuracy defined in section %s with multi=%d",param_name,num_ints);
                        std_error(helpstr);
                }
               if (yscal==(real*) NULL)
                {
                yscal = vector(1,nvar);
                ysav  = vector(1,nvar);
                dysav = vector(1,nvar);
                dym   = vector(1,nvar);
                dyt   = vector(1,nvar);
                yt    = vector(1,nvar);
                ak2 = vector(1,nvar);
                ak3 = vector(1,nvar);
                ak4 = vector(1,nvar);
                ak5 = vector(1,nvar);
                ak6 = vector(1,nvar);
                }
               break;
        case _ReverseEuler:get_flt(param_file,param_name,"relrate",&m_relrate[k]);
                if (m_relrate[k]<=-9999.0) 
                {
                        sprintf(helpstr,"No relrate defined in section %s with multi=%d",param_name,num_ints);
                        std_error(helpstr);
                }
               get_flt(param_file,param_name,"relchange",&m_relchange[k]);
                if (m_relchange[k]<=-9999.0) 
                {
                        sprintf(helpstr,"No relchange defined in section %s with multi=%d",param_name,num_ints);
                        std_error(helpstr);
                }
               yt    = vector(1,nvar);
               dyt   = vector(1,nvar);
               if (oldrate==(real*) NULL)
                {
                oldrate=vector(1,nvar);
                for (j=1;j<=nvar;j++) oldrate[j]=0.0;
                }
               break;
	case _NoInt:break;
        default:printf("Integration method with index %d is set to %d\n",k,m_method[k]);
                       printf("Please selected one of:\n");
                       printf(" %d:  Fixstep \n",_Fixstep);
                       printf(" %d:  Euler  \n",_Euler);
                       printf(" %d:  Runge-Kutta 2nd order, pos. \n",_RK23);
                       printf(" %d:  Runge-Kutta 4th order, pos. \n",_RK45);
                       printf(" %d:  Runge-Kutta Cash-Karp, pos. \n",_RKCK);
                       printf("%d: Runge-Kutta 2nd order \n",-_RK23);
                       printf("%d: Runge-Kutta 4th order \n",-_RK45);
                       printf("%d: Runge-Kutta Cash-Karp \n",-_RKCK);
                       printf(" %d: Reverse Euler \n",_ReverseEuler);
                       printf("%d: No Integration\n",_NoInt);
                       std_error("No such integration method");
        }
     }
 
    if (overrule_on==1) for (k=1;k<=num_ints;k++)
    {
#if defined (__verbose__)
       printf("Overruling integration parameters from %s\n",overrule);
       fflush(stdout);
#endif
        if (num_ints==1)
        {
        sprintf(param_name,"%s",_integ_par_);
        m_assign[1]=-1;
        }
        else
        {
        sprintf(helpname,"%d",k);
        sprintf(param_name,"%s%s",_integ_par_,helpname);
        m_assign[k]=k;
        }

       get_flt(overrule,param_name,"mindelt",&m_mindelt[k]);
       get_flt(overrule,param_name,"maxdelt",&m_maxdelt[k]);
       get_int(overrule,param_name,"method",&m_method[k]);

       switch(abs(m_method[k]))
        {
        case _Fixstep:break;

        case _Euler:get_flt(overrule,param_name,"relrate",&m_relrate[k]);
               get_flt(overrule,param_name,"relchange",&m_relchange[k]);
               if (oldrate==(real*) NULL)
                {
                oldrate=vector(1,nvar);
                for (j=1;j<=nvar;j++) oldrate[j]=0.0;
                }
               break;
        case _RK23:
        case _RK45:
        case _RKCK:get_flt(overrule,param_name,"accuracy",&m_acc[k]);
               if (yscal==(real*) NULL)
                {
                yscal=vector(1,nvar);
                ysav=vector(1,nvar);
                dysav=vector(1,nvar);
                dym=vector(1,nvar);
                dyt=vector(1,nvar);
                yt=vector(1,nvar);
                ak2 = vector(1,nvar);
                ak3 = vector(1,nvar);
                ak4 = vector(1,nvar);
                ak5 = vector(1,nvar);
                ak6 = vector(1,nvar);
               }
               break;
        case _ReverseEuler:get_flt(param_file,param_name,"relrate",&m_relrate[k]);
               get_flt(param_file,param_name,"relchange",&m_relchange[k]);
               if (yt==(real*) NULL)
                {
                yt    = vector(1,nvar);
                dyt   = vector(1,nvar);
		}
               if (oldrate==(real*) NULL)
                {
                oldrate=vector(1,nvar);
                for (j=1;j<=nvar;j++) oldrate[j]=0.0;
                }
               break;
	case _NoInt:break;
        default:printf("Integration method with in section %s is set to %d\n",param_name,m_method[k]);
                       std_error("No such integration method");
        }
       }

/* Dateinamen zusammenbauen */
   sprintf(resultname,"%s",output_file);
   sprintf(output_file,"%s.%s",output_file,resulttype);

/* Result-Typ auswerten; Default ist .outc */
   if (strcmp(resulttype,"nc")==0) netcdf_output=1;   

/* Modell-Directory suchen */
   get_str(param_file,_simu_par_,"model_dir",main_dir);
   sprintf(helpstr,"%s",main_dir);
   for (l=(int)strlen(helpstr);l<__namelength;l++) helpstr[l]= (char) 32;
   helpstr[__namelength]= (char) 0;
   sprintf(main_dir,"%s",helpstr);

/* Result-Directory suchen */
   get_str(param_file,_simu_par_,"result_dir",resultdir);
/* wenn outfile schon mit einem absoluten Pfad angegeben ist, resultdir wieder loeschen */
   if (strpos(output_file,"/")==0) 
   {
       printf("Hint: Resultfile has absolute path; resetting Result-Directory.\n");
       sprintf(resultdir,"%s","");
   }
/* Dateinamen zusammenbauen */
   if (strlen(resultdir)!=0) 
   {
/*  "/" anhaengen */
      sprintf(resultdir,"%s/",resultdir);
/* Dateinamen fuer Result-File zusammenbauen */
      sprintf(helpname,"%s%s",resultdir,output_file);
      sprintf(output_file,"%s",helpname);
/* Dateinamen fuer Info-File zusammenbauen */
      sprintf(helpname,"%s%s",resultdir,info_file);
      sprintf(info_file,"%s",helpname);
      printf("Result-Directory is set. Storing to %s\n",output_file);
      printf("Info-Directory is set. Storing to %s\n",info_file);
   }

#if defined (__with_slave__)
/* Slave Modell-Directory suchen */
   slave_dir[0] = (char) 0;
   get_str(param_file,_slave_,"slave_dir",slave_dir);
/* Die Interface-Box fuer die Kommnikation mit anderen Modellen suchen */
   get_int(param_file,_interface_box_,"connect_box",&interface_box);

/* wenn ein Slave-Modell angegeben ist, werden hier die Kommunikationsstrukturen angelegt */
   if (strlen(slave_dir)>0)
   {
/* Das wollen wir 'mal festhalten */
      i_am_master = 1;
/* Master erzeugt eine neuen Zufallzahlensequenz */
      srand ((unsigned int) getpid());
/* Master erzeugt einen neuen secret_key fuer shared memory Zugriffe */
      secret_key = (key_t) rand() % 1000;

      sprintf (slave_test,"ls %s/cemos-model > /dev/null",slave_dir);
      sprintf (slave_command,"cd %s; ./cemos-model --overrule=./cemos.par --outputfile=slave --resulttype=%s --slave_key=%d > ./slave.log &",slave_dir,resulttype,(int) secret_key);
//       sprintf (slave_command,"cd %s; ddd ./cemos-model",slave_dir);
      sprintf (slave_info,"xterm -title %s/slave.log -e tail -f %s/slave.log &",slave_dir,slave_dir);
   }
/*
Gemeinsamer Code fuer Master und Slave.
Master definiert den Bereich, Slave erhaelt in der Uebergabe den Schluessel und bereitet den Zugriff vor.
*/
#if defined (__verbose__)
   printf("Memory Key %d\n",(int) secret_key);
   fflush(stdout);
#endif
   shmid = shmget (secret_key, sizeof(shared_use_st), 0666 | IPC_CREAT);

   if (shmid == -1)
    {
    std_error("shmget failed");
    }

   shared_memory = shmat(shmid, (void*) 0, 0);
   if (shared_memory == (void*) -1)
    {
    std_error("shmat failed");
    }
#if defined (__verbose__)
   printf("Memory attached at %X\n",(int) shared_memory);
   fflush(stdout);
#endif
   com_cemos = (shared_use_st *) shared_memory;
#endif /* with_slave */

/* Multi-Integration vorbereiten */
  mmdelt=0.0;
  for (k=1;k<=num_ints;k++)
   {
   start_of_int[k]=startim;
   delt_of_int[k]=m_maxdelt[k];
   if (calc_delt>m_maxdelt[k]) calc_delt=m_maxdelt[k];
   if (mmdelt<m_maxdelt[k]) mmdelt=m_maxdelt[k];
   }
  steps= (unsigned long) (0.5+(endtim-startim)/calc_delt);
  for (k=1;k<=num_ints;k++)
   {
   step_len[k]=(unsigned long) (0.5+(m_maxdelt[k]/calc_delt));
   next_int[k]=step_len[k];
   }

/* Anpassung von outdelt, wenn erforderlich */
  if (calc_delt>outdelt)
   {
   outdelt=calc_delt;
   printf("Warning: outdelt adapted to calculation step! New outdelt=%f\n",(float) outdelt);
   }
   store_step=(unsigned long) (0.5+(outdelt/calc_delt));

/*  Zeittakt fuer das Schreiben von *.info */
   if (infofiletime <= 0.0)
   {
/*    In diesem Fall wird nur das Info-File am Ende geschrieben */
      infofiletime = endtim + outdelt;
      infofilestep = endtim;
   }
   else
   {
      infofilestep = infofiletime;
   }
/* Spaeteren Speicherstart vorbereiten */
   start_store=fmax(0.,storetim-startim);

   storetim=fmax(startim,storetim);
/*    printf("storetim= %f\n",storetim); */
   next_store=(unsigned long) (0.5+(start_store/calc_delt));
/*     printf("first store step = %ld\n",next_store); */
/* Startwerte einlesen */
   read_0D_start_values(param_file);
   read_1D_start_values(param_file);
   read_2D_start_values(param_file);

/* Speicherkommando(s) auswerten und Speicherstrukturen vorbereiten */
  num_0D_stores=0;
  total_0D_indexed=num_0D_states+num_0D_globals+num_0D_float_derivs;
  if (total_0D_indexed > 0)
  { 
     get_0D_stores(param_file,total_0D_indexed,&num_0D_stores,&store_0D_ptr,&stores_0D);
     store_0D_vector=fvector(0,num_0D_stores);
     store_0D_names=(name *) malloc(num_0D_stores*sizeof(name));
  }
  num_stores=0;
  total_indexed=num_1D_states+num_1D_globals+num_1D_float_derivs+num_1D_int_derivs;
  if (total_indexed > 0)
  { 
     get_1D_stores(param_file,total_indexed,n_comp,&num_stores,&store_1D_ptr,&stores);
/*  for (i=0;i<num_stores;i++) printf("%s ",stores[i]);*/
  }
  num_stores+=num_0D_stores;
  store_vector=fvector(0,num_stores);
  store_ptr=(real**) malloc((num_stores+1)*sizeof(real*));
  store_names=(name *) malloc((num_stores+1)*sizeof(name));

  num_2D_stores=0;
  total_2D_indexed=num_2D_states+num_2D_globals+num_2D_float_derivs+num_2D_int_derivs;
  if (total_2D_indexed>0) get_2D_stores(param_file,total_2D_indexed,&num_2D_stores,&store_2D_ptr,&stores_2D);
  if (num_2D_stores>0) store_2D_vector=malloc(g_comp1*g_comp2*sizeof(float));

  if (num_2D_stores>0 && netcdf_output==0 )
   {
        printf("Warning: Storage of 2D variable only in netCDF output format. Ignoring 2D storage command!\n");
        fflush(stdout);
        num_2D_stores=0;
   }
 
/* Modulparameter einlesen */
   printf("Reading submodel parameters...\n");
   read__sub(sensitivity);

#if defined (__MULTI_INSTANCES__)
  printf("Removing lock on dynamic paramater files\n");
  fflush(stdout);
  system(__unlock);
#endif

/* Header fuer output file setzen */

  oh.n_stor= num_stores;
  oh.startim = (float) storetim;
  oh.endtim = (float) endtim;
  oh.maxdelt = (float) calc_delt;
  oh.outdelt = (float) outdelt;
  oh.year = (float) year;
  oh.cycle = (float) cycle;
  strncpy(oh.sdir,main_dir,__namelength);

  for (ui=0;ui<num_stores;ui++)
    {
    if (ui<num_0D_stores)
      {
         sprintf(helpstr,"%s",strmid(stores_0D[ui],0,__namelength));
         store_ptr[ui]=store_0D_ptr[ui];
      } else {
         sprintf(helpstr,"%s",strmid(stores[ui-num_0D_stores],0,__namelength));
         store_ptr[ui]=store_1D_ptr[ui-num_0D_stores];
      }
    for (l=(int)strlen(helpstr);l<__namelength;l++) helpstr[l]= (char) 32;
    helpstr[__namelength]= (char) 0;
    sprintf(store_names[ui],"%s",helpstr);
    }
  printf("\n");

if  (netcdf_output==1) {

   status = nc_create(output_file, NC_CLOBBER|NC_SHARE, &ncid);

   if (status != NC_NOERR) {
      sprintf(report,"Could not create output file:%s\n",output_file);
      std_error(report);
   }

   sprintf(helpstr,"created by TIGERGRAPHICS CEMoS");
   ncattput (ncid, NC_GLOBAL, "originator", NC_CHAR, (int) strlen(helpstr),(void *)helpstr);
   status = nc_def_var(ncid, "varnumber",NC_INT,0,dims,&numstor_id);
   if (status != NC_NOERR) {
      sprintf(report,"netCDF-Error when storing header variable %s\n","varnumber");
      std_error(report);
   }
   status = nc_def_var(ncid, "2Dvarnumber",NC_INT,0,dims,&numstor_2D_id);
   if (status != NC_NOERR) {
      sprintf(report,"netCDF-Error when storing header variable %s\n","2Dvarnumber");
      std_error(report);
   }

   status = nc_def_var(ncid, "starttime", NC_FLOAT, 0, dims,&starttime_id);
   if (status != NC_NOERR) {
      sprintf(report,"netCDF-Error when storing header variable %s\n","starttime");
      std_error(report);
   }

   status = nc_def_var (ncid, "endtime", NC_FLOAT, 0, dims,&endtime_id);
   if (status != NC_NOERR) {
      sprintf(report,"netCDF-Error when storing header variable %s\n","endtime");
      std_error(report);
   }

   status = nc_def_var (ncid, "maxdelt", NC_FLOAT, 0, dims,&maxdelt_id);
   if (status != NC_NOERR) {
      sprintf(report,"netCDF-Error when storing header variable %s\n","maxdelt");
      std_error(report);
   }

   status = nc_def_var (ncid, "outdelt", NC_FLOAT, 0, dims,&outdelt_id);
   if (status != NC_NOERR) {
      sprintf(report,"netCDF-Error when storing header variable %s\n","outdelt");
      std_error(report);
   }

   status = nc_def_var (ncid, "year", NC_FLOAT, 0, dims,&year_id);
   if (status != NC_NOERR) {
      sprintf(report,"netCDF-Error when storing header variable %s\n","year");
      std_error(report);
   }

   status = nc_def_var (ncid, "cycle", NC_FLOAT, 0, dims,&cycle_id);
   if (status != NC_NOERR) {
      sprintf(report,"netCDF-Error when storing header variable %s\n","cycle");
      std_error(report);
   }

   status = nc_def_var (ncid, "proceed", NC_FLOAT, 0, dims,&proceed_id);
   if (status != NC_NOERR) {
      sprintf(report,"netCDF-Error when storing header variable %s\n","proceed");
      std_error(report);
   }

   dims[0]= ncdimdef(ncid, "model_directory", __namelength);
   status = nc_def_var (ncid, "model_dir", NC_CHAR, 1, dims,&directory_id);
   if (status != NC_NOERR) {
      sprintf(report,"netCDF-Error when storing header variable %s\n","model_directory");
      std_error(report);
   }

   /* Zeitstempel fuer die Speicherung einrichten */
   dims[0] = ncdimdef(ncid, "time_dim", NC_UNLIMITED);
   timedim=dims[0];
   status = nc_def_var(ncid,"time", NC_FLOAT, 1, dims,&time_id);
   if (status != NC_NOERR) {
      sprintf(report,"netCDF-Error when storing header variable %s\n","time");
      std_error(report);
   }

   /* Variablen einrichten */
   /* Jede zu speichernde Variable wird als Vektor in netCDF angelegt */
   var_id = ivector(0,(long int) num_stores-1);
   for (ui=0;ui<num_stores;ui++)
   {
       strtrim(store_names[ui]);
       status = nc_def_var(ncid,store_names[ui], NC_FLOAT, 1, dims,&var_id[ui]);
       if (status != NC_NOERR) {
          sprintf(report,"netCDF-Error when storing variable %s\n",store_names[ui]);
          std_error(report);
       }
   }
  if (num_2D_stores > 0) 
  {
   _2D_var_id = ivector(0,(long int) num_2D_stores-1);
   netcdf_start_2D[0]=0;
   netcdf_start_2D[1]=0;
   netcdf_start_2D[2]=0;
/*
Fuer die korrekte Darstellung (Matrix-Sortierung) in ncview muss hier die Y-Dimension in netcdf_count_2D[1] geschrieben werden.
Steuerung via NC_FLIP
 
Noch nicht komplett verstanden aber geht. Siehe auch unten bei dims_2D (Reihenfolgen muessen passen!)
*/
   netcdf_count_2D[0]=1;
#if defined (NC_FLIP)
   netcdf_count_2D[2]=g_comp2;
   netcdf_count_2D[1]=g_comp1;
   status = nc_def_dim(ncid,"g_comp1",g_comp1, &dim2_2D);
   if (status != NC_NOERR) {
       sprintf(report,"netCDF-Error when storing variable %s\n","dim2_2D");
       std_error(report);
   }
   status = nc_def_dim(ncid,"g_comp2",g_comp2, &dim1_2D);
   if (status != NC_NOERR) {
       sprintf(report,"netCDF-Error when storing variable %s\n","dim1_2D");
       std_error(report);
   }
#else
   netcdf_count_2D[1]=g_comp2;
   netcdf_count_2D[2]=g_comp1;
   status = nc_def_dim(ncid,"g_comp1",g_comp1, &dim1_2D);
   if (status != NC_NOERR) {
       sprintf(report,"netCDF-Error when storing variable %s\n","dim1_2D");
       std_error(report);
   }
   status = nc_def_dim(ncid,"g_comp2",g_comp2, &dim2_2D);
   if (status != NC_NOERR) {
       sprintf(report,"netCDF-Error when storing variable %s\n","dim2_2D");
       std_error(report);
   }

#endif

   dims_2D[0]=timedim;
   dims_2D[1]=dim2_2D;
   dims_2D[2]=dim1_2D;

   for (ui=0;ui<num_2D_stores;ui++)
   {
       strtrim(stores_2D[ui]);
       sprintf(helpstr,"%s",stores_2D[ui]);
       status = nc_def_var(ncid,helpstr, NC_FLOAT, 3, dims_2D,&_2D_var_id[ui]);
       if (status != NC_NOERR) {
          sprintf(report,"netCDF-Error when storing variable %s\n",stores_2D[ui]);
          std_error(report);
       }
   }
 }


   ncendef (ncid);

   ncvarput(ncid,numstor_id,netcdf_start,netcdf_stop,(void *)&oh.n_stor);
   ncvarput(ncid,starttime_id,netcdf_start,netcdf_stop,(void *)&oh.startim);
   ncvarput(ncid,endtime_id,netcdf_start,netcdf_stop,(void *)&oh.endtim);
   ncvarput(ncid,maxdelt_id,netcdf_start,netcdf_stop,(void *)&oh.maxdelt);
   ncvarput(ncid,outdelt_id,netcdf_start,netcdf_stop,(void *)&oh.outdelt);
   ncvarput(ncid,year_id,netcdf_start,netcdf_stop,(void *)&oh.year);
   ncvarput(ncid,cycle_id,netcdf_start,netcdf_stop,(void *)&oh.cycle);

   ncvarput(ncid,numstor_2D_id,netcdf_start,netcdf_stop,(void *)&num_2D_stores);

   netcdf_start[0]=0;
   netcdf_stop[0]=__namelength;
   ncvarput(ncid,directory_id,netcdf_start,netcdf_stop,(void *)&oh.sdir);
/*    nc_close(ncid); */

} else {
  if ((outfile=fopen(output_file,"wb")) == (FILE *) NULL )
  {
   sprintf(report,"Cannot open : %s \n Read only file (system)?\n",output_file);
   std_error(report);
  }
  if (strcmp(resulttype,"outa") == 0) 
  {
	  fprintf(outfile,"Stored Variables:%u\n", oh.n_stor);
	  fprintf(outfile,"Start Time:%f\n",oh.startim);
	  fprintf(outfile,"End Time:%f\n",oh.endtim);
//	  fprintf(outfile,"Maxdelt:%f\n",oh.maxdelt);
	  fprintf(outfile,"Outdelt:%f\n",oh.outdelt);
	  fprintf(outfile,"Year:%f\n",oh.year);
	  fprintf(outfile,"Cycle:%f\n",oh.cycle);
//	  fprintf(outfile,"ModelDir:%s\n",oh.sdir);

	  fprintf(outfile,"time,");
	  for (ui=0;ui<num_stores-1;ui++) 
	  {
		  strtrim(store_names[ui]);
		  fprintf(outfile,"%s,",store_names[ui]);
	  }
	  strtrim(store_names[num_stores-1]);
      fprintf(outfile,"%s\n",store_names[num_stores-1]);
    } else {
	  fwrite(&oh,sizeof(oh),1,outfile);
	  for (ui=0;ui<num_stores;ui++) fwrite(&store_names[ui],__namelength,1,outfile);
  }
}
/* Informationen fuer das Log-file ausgeben   */
  ui=sizeof(real);
  if (ui==sizeof(float)) printf("Starting simulation with single precision numerics length=%d \n",(int) sizeof(real));
  if (ui==sizeof(double)) printf("Starting simulation with double precision numerics length=%d \n",(int) sizeof(real));
  if (ui==sizeof(long double)) printf("Starting simulation with long double numerics length=%d \n",(int) sizeof(real));
  fflush(stdout);
  printf("=====================================================\n");
  printf("Running simulation with integration parameters:\n");
  printf("starttime= %f\n",(float) startim);
  printf("endtime= %f\n",(float) endtim);
  printf("outdelt= %f\n",(float) outdelt);
  printf("year= %f\n",(float) year);
  printf("cycle= %f\n",(float) cycle);
  printf("storetime= %f\n",(float) storetim);
  printf("multi= %d\n",num_ints);
  printf("resulttype= %s\n",resulttype);
  printf("recalc_globals= %d\n",recalc_globals);
  printf("=====================================================\n");

  for (k=1;k<=num_ints;k++)
  {
  printf("Integration parameters for process %d\n",k);
  printf("method[%d]= %d\n",k,m_method[k] );
  printf("mindelt[%d]= %g\n",k,(float) m_mindelt[k]);
  printf("maxdelt[%d]= %g\n",k,(float) m_maxdelt[k]);

    switch(abs(m_method[k]))
     {
     case _Fixstep:break;

     case _Euler:printf("relrate[%d]= %g\n",k,(float) m_relrate[k]);
            printf("relchange[%d]= %g\n",k,(float) m_relchange[k]);
            break;
     case _RK23:
     case _RK45:
     case _RKCK:printf("accuracy[%d]= %g\n",k,(float) m_acc[k]);
            break;
     case _ReverseEuler:printf("relrate[%d]= %g\n",k,(float) m_relrate[k]);
            printf("relchange[%d]= %g\n",k,(float) m_relchange[k]);
            break;
     default:break;
     }
  printf("=====================================================\n");
  }

/* Modell einmal ausfuehren, um die globalen Variablen zu setzen */
  sim_time=startim;
  int_time=(unsigned int) startim;
  if (cycle>0.0) day_of_year=int_time%(int) (cycle); else day_of_year=int_time;
  assign=-1;
  maxdelt=calc_delt;

  model();
  make_global_0D_states();
#if defined (__with_slave__)
/* Slave starten */
  if (i_am_master == 1)
   {
   com_cemos->master_pid = getpid();
   com_cemos->master_is_running = 1;
   com_cemos->requested_time = (double) 0.0;

   printf("Starting SLAVE model at %s\n",slave_dir);
   fflush(stdout);

   j = system(slave_test);
   if (j != 0) std_error("Slave Model not available");

   system(slave_command);
   system(slave_info);
/* Auf Slave warten */
   while (com_cemos->slave_is_running == 0);
   printf("SLAVE model running with pid %d\n",com_cemos->slave_pid);
   fflush(stdout);

   while (com_cemos->reached_time < com_cemos->requested_time);
   }

/* SLAVE meldet, dass die Simulation jetzt startet */
    if (i_am_slave  == 1)
     {
     com_cemos->slave_pid=getpid();
     com_cemos->slave_is_running = 1;
     com_cemos->reached_time = (double) -1.0;
     }
#endif

#if defined(__ljapunov__)
  if (liap==1) ljapunov();
#endif

  time(&start);

/* Zeitschleife fuer die Simulation */
  for( step=0; step <= steps+1; step++)
    {
    startk=startim + calc_delt * (real) (step);
/* Aufpassen, dass das Modell nicht doch zu weit laeufft */
    if (startk-startim > ((real) steps)*outdelt) break;

    end_of_int=startim + calc_delt * (real) (step+1);
/* Maximale Simulationszeit auf Endtime begrenzen
    if (end_of_int > endtim) end_of_int = endtim;*/

    int_time=(unsigned int) startk;
    if (cycle>0.0) day_of_year= int_time%(int) (cycle); else day_of_year=int_time;

#if defined (__with_slave__)
/* Kommunikationsinterface fuer Master-Modelle*/
     if (i_am_master==1)
      {
/* Empfangen der Ergebnisse von Slave */
       if (interface_box > 0) master_reads_slave(interface_box);

/* Simulationszeit setzen, fuer die SLAVE das naechste Zwischenergebnis bereitstellen soll */
       com_cemos->requested_time = (double) startk;
//       kill (com_cemos->slave_pid,SIGCONT);

/* ... und warten, bis es weitergehen kann ... */
       while (com_cemos->reached_time < com_cemos->requested_time)
        {
// 	kill (com_cemos->master_pid,SIGSTOP);
/*
	printf("Master continues...\n");
	fflush(stdout);
*/
	}
/*
       {
       printf("Waiting for SLAVE at time %f with reached time %f\n",com_cemos->requested_time,com_cemos->reached_time);
       fflush(stdout);
       }
*/
/* hier kann dann die Uebertragung der Ergebnisse aus Slave nach Master erfolgen */

      }

/* Kommunikationsinterface fuer Slave-Modell */
     if (i_am_slave==1)
      {
/* Senden der Ergebnisse nach Master */
       if (interface_box > 0) slave_sets_master(interface_box);

/* Master mitteilen, welche Simulationszeit bis jetzt erreicht wurde */
       com_cemos->reached_time = (double) startk;
//        kill (com_cemos->master_pid,SIGCONT);

/* ... und warten, bis die Fortsetzung angefordert wird ... */
       while (com_cemos->reached_time >= com_cemos->requested_time)
        {
// 	kill (com_cemos->slave_pid,SIGSTOP);
/*
	printf("Slave continues...\n");
	fflush(stdout);
*/
	}

/*
       {
       printf("Waiting for MASTER at time %f with request for time %f\n",com_cemos->reached_time,com_cemos->requested_time);
       fflush(stdout);
       }
*/
      }
#endif



    if (startk>=info_time)
     {
#if defined (__verbose__)
     printf("Simulation time %f \n",(float) startk);
     fflush(stdout);
#endif     
     info_time+=info_step;
     }

    if (startk>=infofiletime)
     {
     sprintf(info_filename,"%s%f.info",info_file,(float) startk);
     write_info(info_filename);
     infofiletime += infofilestep;
     }
/* Speichern vor der Integration, wenn storestart==0  */
    if (step==next_store && storestart==0)
     {
/*Der Programmblock "store_sequence.c" wird an zwei Stellen identisch benötigt. Die ein ist hier; die andere bei storestart!=0
Das Code-Segment liegt in "CEMOS/include"*/
#include "store_sequence.c"
     }


    for(k=1;k<=num_ints;k++)
     {
     if ((step+1)==next_int[k])
      {
      assign=m_assign[k];
      method=m_method[k];
      acc=m_acc[k];
      relrate=m_relrate[k];
      relchange=m_relchange[k];
      maxdelt=m_maxdelt[k];
      mindelt=m_mindelt[k];

/*      printf("%d %g %g\n",assign,start_of_int[k],end_of_int);*/


      switch(method)
       {
       case  _Fixstep:fixstep(start_of_int[k],delt_of_int[k]);
               break;
       case -_Euler:
       case  _Euler:euler(start_of_int[k],end_of_int,&delt_of_int[k]);
               break;
        case -_RK23:
        case -_RK45:
        case  _RK23:
        case  _RK45:
        case -_RKCK:
        case  _RKCK:odeint(start_of_int[k],end_of_int,&delt_of_int[k]);
/* Die Fluesse nocheinmal rechnen, damit sie zu den States passen, aber nur wenn dies gefordert ist.... */
               if (recalc_globals==1) no_integration(end_of_int,maxdelt);
               break;
       case -_ReverseEuler:
       case  _ReverseEuler:back_euler(start_of_int[k],end_of_int,&delt_of_int[k]);
               break;
       case -_NoInt:
       case  _NoInt:no_integration(start_of_int[k],delt_of_int[k]);
                    break;
       }

      start_of_int[k]=end_of_int;
      next_int[k]+=step_len[k];
      }
     }

#if defined(__ljapunov__)
     if (liap==1) ljapunov();
#endif

/* Speichern nach der Integration, wenn storestart!=0  */
    if (step==next_store && storestart!=0)
     {
/*Der Programmblock "store_sequence.c" wird an zwei Stellen identisch benötigt. Die ein ist hier; die andere bei storestart==0
Das Code-Segment liegt in "CEMOS/include"*/
#include "store_sequence.c"
     }

    }
if (netcdf_output==1) {
    nc_close(ncid);
} else {
    fclose(outfile);
}

    time(&end);
    sprintf(info_filename,"%s.info",info_file);
    write_info(info_filename);

    printf("\nSimulation results written to %s\n",output_file);

    printf("Total time: %u seconds \n",(unsigned)(end-tstart));
    printf("Simulation time: %u seconds \n",(unsigned)(end-start));

#if defined (__with_slave__)

/* Fuer den Fall, dass der Masterprozess schneller fertig ist als der Slave, wird letzter abgeschossen */

    if (i_am_master == 1)
     {
     if (com_cemos->slave_is_running==1)
      {
       printf("SLAVE Process killed by MASTER\n");
       kill( (pid_t) com_cemos->slave_pid, SIGKILL);
      }
     }
/* Fuer den Fall, dass der Slaveprozess schneller fertig ist als der Master, wird letzter abgeschossen */
    if (i_am_slave == 1)
     {
     if (com_cemos->master_is_running==1)
      {
      printf("MASTER Process killed by SLAVE\n");
      kill( (pid_t) com_cemos->master_pid, SIGKILL);
      }
     }

/* und zum Schluss wird das shared memory freigegeben, von wem auch immer ;-) */
    if (shmdt(shared_memory) == -1)
     {
     std_error("shmdt failed");
     }

    if (shmctl(shmid,IPC_RMID,0) == -1)
     {
     std_error("shmid failed");
     }
#endif
system("rm -f cemos.pid");
return(0);
}


