# CEMOS/CEMTK 5.0 Copyright(C) 2022 Cora Kohlmeier, Frank Hamberg
proc compile {mpath gflags cflags lflags} {

global GUI DOS env cemoshome pwd netCDFHOME arch plat
global processready
global MULTI_INSTANCES

set NULL {}
set SLAVE 0

set pattern "-g"
if { $DOS==0 } {
   set struct_flags "-Wredundant-decls"
} else {
   set struct_flags "-D__ignore_fminmax"
}
set dbf 0

if ![info exist MULTI_INSTANCES] {set  MULTI_INSTANCES 0}
if {$MULTI_INSTANCES==0} {
   set MUI ""
} else {
    set MUI -D__MULTI_INSTANCES__
}

foreach item $gflags {
      if [string match $pattern $item] {set dbf 1}
}

if {$plat=="arm64"} {
	set CC "gcc -target arm64-apple-macos"
} else {
	set CC gcc
}	
	
set preprocess "$CC -E"

set model_dir {}
set STATUS {}

set model_dir  $mpath
set model_path $pwd

if {$model_dir != $NULL} {
   set model_path $pwd/$model_dir
}

set MODEL $model_path
if ![file isdirectory $MODEL] {
   puts "Error no such directory: $MODEL"
   return
}

clean $model_dir

cd $model_path
if [file exists install.tcl] {
   put "Executing install script"
   set cmd "exec bash ./install.tcl u"
   catch $cmd help
   put $help
   set cmd "exec  bash ./install.tcl a"
   catch $cmd help
   put $help
put "Install script ready"

}

set list_sub [myls $model_path *.def]

set file [myls $model_path *model.def]
set tempfile $model_path/model.tmp
set tempcfile $model_path/modelpar.c

if {[llength $file] > 1} {
  put "More than one file named *model.def !!"
  put "Rename the submodel file !"
  return
}

set index [lsearch -exact $list_sub $file]
set list_sub [lreplace $list_sub $index $index]

set model_source [myls $model_path *.c]

set str_ext .c
set pr_ext _prae.c
set hdr_ext .h
set prp_ext _prae.i

set struct_def [open struct_def.c w]
set struct_header [open struct.h w]
set type_def [open precision.h w]
set setup_def [open compiler_setup.h w]

switch $DOS {
    0 { puts $type_def "#define LINUX" }
    1 { puts $type_def "#define DOS" }
    2 {puts $type_def "#define DARWIN"}
    default { puts $type_def "#define LINUX"}
}
copy $file $tempcfile

set cmd "eval exec $preprocess $tempcfile > $tempfile"
catch $cmd STATUS

if {$STATUS != {}} {
    put "$STATUS caused by $file"
#      put "ERROR"
#      return
}

set precision "typedef float real;"

  set id [open $tempfile r]
  set prec [string match *numeric?double* [read $id]]
  close $id
   if {$prec != 0} {
      set precision "typedef double real;"
   }
  set id [open $tempfile r]
  set prec [string match *numeric?long* [read $id]]
  close $id
   if {$prec != 0} {
      set precision "typedef long double real;"
   }

puts $type_def $precision

  set id [open $tempfile r]
  set slave [string match *%with_slave* [read $id]]
  close $id
   if {$slave != 0 && ($DOS==0 || $DOS==2)} {
      put "Compiling for use of slave model"
      puts $type_def "#define __with_slave__"
      regsub -all -- {-O[1-9]} $cflags {-O0} cflags
      put "Switching off optimisation"
   }

  set setup ""
  set id [open $tempfile r]
  foreach line [split [read $id] \n] {
  set defsetup [string match *%setup* $line]
  if {$defsetup != 0} {
      set setup [lindex [split $line] 1]
      puts $setup_def "#define $setup"
      put "Compiling model for setup $setup"  
   }
  }
  close $id


remove $tempfile
remove $tempcfile

close $type_def
close $setup_def

set INCLUDE_PATH "-I$cemoshome/include -I$model_path -I$netCDFHOME/include"

put "Building cemos-model with $MODEL"

cd $model_path

puts $struct_def "#include \"types.h\"" 
puts $struct_def "void struct_def(string sensitivity) {"


set name [file rootname $file]

set pr_file $name$pr_ext
set ppr_file $name$prp_ext

cat3 precision.h compiler_setup.h $file $pr_file

set cmd "eval exec $preprocess $pr_file > $ppr_file"
catch $cmd STATUS

if {$STATUS != {}} {
  put "$STATUS caused by $file"
}

set struct struct_$name$str_ext
set header struct_$name$hdr_ext
set para_file parfile.tmp
set define $ppr_file
set reset_derivs reset_derivs.c

put " "
put "Creating and compiling the structure-builders...."

if {$DOS==0 || $DOS==2} {
    set target $cemoshome/prepare
} else {
    set target $cemoshome/prepare.exe
}

set cmd "eval exec $target $define $struct $header \
            struct_$name $pwd $para_file \
            $model_path $reset_derivs $pwd/.sensitivity.def $model_path/$file"
catch $cmd STATUS

if {$STATUS != {}} {
  put $STATUS
  put "ERROR during execution of prepare"
  return
}

set help "$pr_file $ppr_file"
remove $help

# Compileraufruf

  put "$CC $gflags -c $struct"
  set cmd "eval exec $CC $gflags $INCLUDE_PATH -c $struct"
  catch $cmd STATUS

  if {$STATUS != {}} {
      put $STATUS
#      put "ERROR"
#      return
  }


if {$dbf != 1} {
  remove $struct
}

puts  $struct_def "void struct_$name (string);"

puts $struct_header "\#include \"struct_$name$hdr_ext\""

close $struct_header

eval exec $CC  $gflags $cflags -D__ignore_fminmax $INCLUDE_PATH -c $reset_derivs

if {$dbf != 1} {
  remove $reset_derivs
}

set name [file rootname $file]

puts  $struct_def "struct_$name (sensitivity);"
puts  $struct_def }
close $struct_def

# Erzeugen der Strukturen fuer die einzelnen Module

set read_sub [open read__sub.c w]
#----------------------------------------------------------------------------------------------------------------
puts $read_sub "#include \"types.h\"" 

foreach file $list_sub {
   set name [file rootname $file]

   set pr_file $name$pr_ext
   set ppr_file $name$prp_ext
   cat3 precision.h  compiler_setup.h $file $pr_file

   set cmd "eval exec $preprocess $pr_file > $ppr_file"
   catch $cmd STATUS

   if {$STATUS != {}} {
      put "$STATUS caused by $file"
#      put "ERROR"
#      return
   }

   set struct read__$name$str_ext
   set header $name$hdr_ext
   set define1 $file
   set define $ppr_file

   if {$DOS==0 || $DOS==2} {
       set target $cemoshome/prepare_sub
   } else {
       set target $cemoshome/prepare_sub.exe
   }


  set cmd "eval exec $target $define $struct $header \
                 read__$name $model_path $pwd $model_path/$define1  $pwd/.sensitivity.def $define1"


   catch $cmd STATUS

   if {$STATUS != {}} {
      put $STATUS
      put "ERROR during execution of prepare_sub for $define1"
      return
   }

   remove $pr_file
   remove $ppr_file

# Compileraufruf

   put "$CC $gflags -c $struct"
   eval exec $CC $struct_flags $gflags $INCLUDE_PATH -c $struct

   if {$dbf != 1} {
      remove $struct
   }

   puts $read_sub "extern void read__$name (string);"
}
# end foreach file in list_sub
#----------------------------------------------------------------------------------------------------------------


puts $read_sub "void read__sub(string sensitivity) {"
foreach file $list_sub {
   set name [file rootname $file]
   puts $read_sub "read__$name (sensitivity);"
}
puts $read_sub }
close $read_sub

cd $model_path

# Compileraufruf

put "$CC  $gflags -c read__sub.c"
eval exec $CC  $struct_flags $gflags $INCLUDE_PATH -c read__sub.c

if {$dbf != 1} {
 remove read__sub.c
}

# Compileraufruf

put "$CC  $gflags -c struct_def.c"
eval exec $CC  $gflags $INCLUDE_PATH -c struct_def.c
if {$dbf != 1} {
 remove struct_def.c
}



cd $cemoshome/frame
set list [myls [pwd] *.c]

copy $list $model_path

cd $cemoshome/util
set help_list [exec ls]
set help_list [myls [pwd] *.c]
copy $help_list $model_path
set list [concat $list $help_list]

cd $cemoshome/driver
set help_list [myls [pwd] *.c]
copy $help_list $model_path
set list [concat $list $help_list]

cd $model_path
# Compileraufruf

put " "
put "Compiling the model files........"

foreach file $model_source {
  if [file exists $file] {
    put "$CC $gflags $cflags -c $file"

     set cmd "eval exec $CC $gflags $cflags $INCLUDE_PATH -c $file"
     catch $cmd STATUS

    if {$STATUS != {}} {
       put $STATUS
     put "*****************************************************************"
#       put "ERROR: Correct the C-code in this file!"
#       return
    }
  }
}
put " "
put "Compiling the CEMoS-package files " 0
# dies sind Zusatzflsgs, die die warnungsfreie Nutzung der sorgfältig 
# überprüften String-Operationen (sprintf, strncpy, etc) im CeMoS-Paket erlauben
set cemosflags "-Wno-format-overflow -Wno-restrict -Wno-stringop-truncation"
set cemosflags ""
foreach file $list {
  if [file exists $file] {
   put "." 0
#    put "$CC $gflags $cflags $MUI -c $file"
    set cmd "eval exec $CC $gflags $cflags $cemosflags $MUI $INCLUDE_PATH -c $file"
    catch $cmd STATUS

    if {$STATUS != {}} {
       put $STATUS
     put "*****************************************************************"
#       put "ERROR: Correct the C-code in this file!"
#       return
    }

   if {$dbf != 1} {
     remove $file
   }
  }
}

put " "
set list [myls [pwd] *.o]

set cmd "eval exec ar rv cemos.a $list"
catch $cmd STATUS
#put $STATUS

exec ranlib cemos.a
put " "
put "Creating new executable...."

if {$DOS==0 || $DOS==2} {
   put "$CC -o cemos-model cemos.a -L$netCDFHOME/lib -lm $lflags"
   set cmd "eval exec $CC -o cemos-model cemos.a -L$netCDFHOME/lib -lm $lflags"
   
   catch $cmd STATUS
   put $STATUS
   put "Removing object-files...."
   remove $list
   move cemos-model $pwd
} else {
   set cmd "eval exec $CC -o cemos-model.exe $list -L$netCDFHOME/lib -lm -lnetcdf"
   catch $cmd STATUS
   put $STATUS
   put "Removing object-files...."
   remove $list
   
   move cemos-model.exe $pwd
}  

if {$STATUS=={} } {
      if {$GUI==1} {activate_start}
      if {$setup != ""} {
         put "Model is ready for setup $setup"  
      } else {
         put "Model is ready........."
      }
   }   

cd $pwd


}
