/* MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg */

#include "stdinc.h"
#include "matrix.h"

void data_analyse(int harmonics,float *varianz, int num_data, real *time_vec,real *var_dat,real *var_coeff)
{
int base_vecs,i,j,k,n;

real **x_matrix, **d_matrix, **r_matrix,**x_matrix_trsp,**dx_matrix,**xx_matrix,**calc_matrix;
real *fcoeff,*base_data;
real val_mean;
real pi_1,argu,epsi;

base_vecs = IMIN(2*harmonics,IMAX(2,(int) (2*num_data/2)-1));

fcoeff = vector(0,base_vecs);

if (num_data == 1) val_mean = var_dat[1];

if (num_data > 1)
{
   val_mean = average(var_dat,1, num_data);
   if (*varianz <= 0.0)
   {
     epsi = stddev(var_dat,val_mean,1,num_data);
     if (fabs(val_mean) > 1.0e-3)  epsi /= val_mean;
     epsi *= epsi;
     *varianz = epsi;
   }
   else
   {
     epsi = (real) *varianz;
   }

/* Matrix zum Rechnen und ihre Transponierten*/
   x_matrix =		matrix(1,num_data,0,base_vecs-1);
   x_matrix_trsp= 	matrix(0,base_vecs-1,1,num_data);
   d_matrix =		matrix(0,base_vecs-1,0,base_vecs-1);
   dx_matrix =		matrix(0,base_vecs-1,1,num_data);
   xx_matrix =		matrix(1,num_data,1,num_data);
   r_matrix =		matrix(1,num_data,1,num_data);
   calc_matrix=		matrix(0,base_vecs-1,1,num_data);
   base_data  =		vector(1,num_data);
printf("Allocated ");
/*Matrizen vorbereiten*/
   for (i=1;i<=num_data;i++) for(j=0;j<base_vecs;j++) x_matrix[i][j] = 0.0;
   for (i=0;i< base_vecs;i++) for(j=0;j<base_vecs;j++) d_matrix[i][j] = 0.0;
   for (i=1;i<=num_data;i++) for(j=1;j<=num_data;j++)   if (i==j) r_matrix[i][j] = epsi; else r_matrix[i][j]=0.0;
printf("Initialised ");
/* Matrix mit den neuen Basis-Vektoren fllen */
  for (n=1;n<=base_vecs/2;n++)
     {
     pi_1 = 1.0 / (M_PI  * n *n);
     d_matrix[2*n-2][2*n-2] = pi_1;
     d_matrix[2*n-1][2*n-1] = pi_1;

     for (k=1;k<=num_data;k++)
         {
	 argu = n * time_vec[k];
	 x_matrix[k][2*n-2] = cos(argu);
	 x_matrix[k][2*n-1] = sin(argu);
	 }
     }
printf("0 ");
fflush(stdout);
/* Messwerte auf den Mittelwert verschieben */
     for (k=1;k<=num_data;k++) base_data[k] = var_dat[k] - val_mean;

/*x_matrix transponieren     */
    matr_transpose(x_matrix,x_matrix_trsp,1,num_data,0,base_vecs-1);
printf("1 ");
fflush(stdout);
/*dx_matrix erzeugen*/
    matr_matr_mul(d_matrix,x_matrix_trsp,dx_matrix,0,base_vecs-1,0,base_vecs-1,0,base_vecs-1,1,num_data);
printf("2 ");
fflush(stdout);
/* Quadratische Zwischenmatrix erzeugen */
    matr_matr_mul(x_matrix,dx_matrix,xx_matrix,1,num_data,0,base_vecs-1,0,base_vecs-1,1,num_data);
/*r_matrix zu xx_matrix addieren, Ergebnis xx_matrix */
printf("3 ");
fflush(stdout);
    matr_matr_add(xx_matrix,r_matrix,xx_matrix,1,num_data,1,num_data,1,num_data,1,num_data);
/*xx_matrix invertieren*/
printf("4 ");
fflush(stdout);
    matr_inversion(xx_matrix,xx_matrix,1,num_data);
/*endgltige Transformationsmatrix erzeugen*/
printf("5 ");
fflush(stdout);
    matr_matr_mul(dx_matrix,xx_matrix,calc_matrix,0,base_vecs-1,1,num_data,1,num_data,1,num_data);
/* Koeffizienten berechnen */
printf("6 ");
fflush(stdout);

matr_vect_mult(calc_matrix,base_data,var_coeff,0,base_vecs-1,1,num_data,1,num_data,1,base_vecs);
printf("ok ");
fflush(stdout);

/* Matrizen und Vektoren freigeben */
 free_matrix(x_matrix,1,num_data,0,base_vecs-1);
 free_matrix(x_matrix_trsp,0,base_vecs-1,1,num_data);
 free_matrix(d_matrix,0,base_vecs-1,0,base_vecs-1);
 free_matrix(dx_matrix,0,base_vecs-1,1,num_data);
 free_matrix(xx_matrix,1,num_data,1,num_data);
 free_matrix(r_matrix,1,num_data,1,num_data);
 free_matrix(calc_matrix,0,base_vecs-1,1,num_data);
 free_vector(base_data,1,num_data);
 free_vector(fcoeff,0,base_vecs);
 }

/*Mittelwert als nullte Spektrallinie schreiben */
   var_coeff[0] = val_mean;



}





