/* MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg */

#include "stdinc.h"
#include "matrix.h"

int main(int argc, char *argv[])
{
 void *readheader(FILE*,header*,int);
 void writeheader(FILE *, header*, long_name80*, int);


 char *resultname;
 char *HOME;
 string input_file,output_file,transform_file;
 long int timnum;
 int i,j,k,l,n,m, namelength, namelength1;
 int base_vecs;
 float num_of_cycles;
  long_name  one_name;
  long_name80 *all_var_names;
  string helpname,para;
  void *names;
  float *dat;
  float set_cycle=-1.0,set_outdelt=-1.0;
  real argu;
  real **all_data,**target_data;
  real **x_matrix, **xx_matrix, **x_matrix_trsp, **y_matrix_trsp;
  real *var_coeff, *base_data;

/* Normally the programm is called as "transform_result" and function is set to 0. If the programm is called as "correlation" function is set to 1  */
  int function=0;

  FILE *in_file, *out_file, *transform;

  header oh1,oh2;

  namelength=sizeof(long_name80);
  oh2.sdir = malloc(sizeof(char)*namelength);

  HOME=getenv("HOME");
  
  if (strpos(argv[0],"correlation")==0) function=1;

  resultname=argv[1];

  if (argc > 1)
  {
  /*   Vorgabe fuer cycle einlesen    */
     sprintf(para,"%s",argv[2]);
     sscanf(para,"%f",&set_cycle);

/*   Vorgabe fuer outdelt einlesen    */
     sprintf(para,"%s",argv[3]);
     sscanf(para,"%f",&set_outdelt);
  }

  if (strpos(resultname,".outb") > 0) {
    namelength1 = sizeof(name12);
  } else {
    namelength1 = sizeof(long_name80);
  }

  sprintf(input_file,"%s",resultname);

  /*    Ausgabenamen zusammenbauen und dabei die Extension ".outb" oder ".outc" abschneiden */
  i=strpos(resultname,".out");
  if (function==1)
  {
      sprintf(output_file,"%s_autocorr_pk.outc",strmid(resultname,0,i));
  }
  else
  {
      sprintf(output_file,"%s_pk.outc",strmid(resultname,0,i));
  }

  in_file=fopen(input_file,"r");

  if (in_file == NULL)
  {
     std_error("Error when opening data file for reading");
  }
/* Header der Quelldatei einlesen */
  names = readheader(in_file,&oh1,namelength1);

/* cycle übersteuern, wenn nicht default (-1.0) gewählt übergeben wurde */
  if (set_cycle > 0.0)
  {
     oh1.cycle  = set_cycle;
  }

 /*Variable-Liste fuer die Ausgabedatei zusammenbauen*/
 all_var_names = (long_name80*) malloc(sizeof(long_name80)*oh1.nvars);
  for (i=0;i<oh1.nvars;i++)
  {
     sprintf(helpname,"%s",strmid(names,i*namelength1,namelength1));
/*      printf("%d:|%s|",i,helpname); */
     for (m=(int)strlen(helpname);m<namelength;m++) helpname[m]= (char) 32;
     sprintf(all_var_names[i],"%s",strmid(helpname,0,namelength));
/*      printf("|%s|\n",all_var_names[i]); */
  }

  timnum = (long int) (0.5+(oh1.endtim-oh1.startim)/oh1.outdelt) +1;
  num_of_cycles = (int) (.5+(oh1.endtim-oh1.startim)/oh1.cycle);

  printf("Time information of input data:\n");
  printf("Starttime %f, Endtime: %f, Cycle: %f, Number of cycles %f, Number of time steps: %ld \n",
          oh1.startim,oh1.endtim, oh1.cycle,num_of_cycles,timnum);
  fflush(stdout);

/* Daten fuer Zieldatei erzeugen und Fourieranalyse vorbereiten */

/* Vorgabewerte fuer die Fourier-Transformation */
  base_vecs = 2 *  (int) (timnum / 2.0);
  printf("Number of harmonics set to %d\n",base_vecs/2);
  fflush(stdout);

/* Feld fuer Spektren anlegen */
  target_data = matrix(1,oh1.nvars,0,base_vecs);
  var_coeff   = vector(0,base_vecs);

/*
Ueberpruefen, ob fuer die vorliegenden Werte "timnum" und "base_vecs"schon eine Transformationsmatrix vorliegt.
In disem Fall wird die Transformationsmatrix direkt eingelesen,
andernfalls wird sie berechnet und anschliessend fuer's Recycling weggeschrieben
*/

/* Dateinamen erzeugen */
sprintf(transform_file,"%s/.MOVIE-matrices/transmatrix%ld%d.dat",HOME,timnum-1,base_vecs);

transform=fopen(transform_file,"rb");
if (transform == NULL)
{
   x_matrix =      matrix(1,timnum,0,base_vecs-1);
   x_matrix_trsp = matrix(0,base_vecs-1,1,timnum);
   /* Matrix mit den neuen Basis-Vektoren fuellen */
   printf("Preparing matrices; stage: 0 ");
   fflush(stdout);

     for (n=1;n<=base_vecs/2;n++)
        {
        for (k=1;k<=timnum;k++)
            {
	    argu = 2.0 * M_PI * n * (k-1) / timnum;
	    x_matrix[k][2*n-2] = cos(argu);
	    x_matrix[k][2*n-1] = sin(argu);
	    }
        }

/* x_matrix transponieren; das Ergebnis steht dann in x_matr_trsp */
   printf("1 ");
   fflush(stdout);
   matr_transpose(x_matrix,x_matrix_trsp,1,timnum,0,base_vecs-1);

/* Quadratische Zwischenmatrix erzeugen */
   printf("2 ");
   fflush(stdout);
   xx_matrix = matrix(0,base_vecs-1,0,base_vecs-1);
   matr_matr_mul(x_matrix_trsp,x_matrix,xx_matrix,0,base_vecs-1,1,timnum,1,timnum, 0,base_vecs-1);
/* x_matrix wird nicht mehr gebraucht */
   free_matrix(x_matrix,1,timnum,0,base_vecs-1);

/* xx_matrix invertieren */
   printf("3 ");
   fflush(stdout);
   matr_inversion(xx_matrix,xx_matrix,0,base_vecs-1);
/* und nun die vorlaeufig endgueltige Version von x_matrix erzeugen */
   printf("4 ");
   fflush(stdout);

   y_matrix_trsp = matrix(0,base_vecs-1,1,timnum);
   matr_matr_mul(xx_matrix,x_matrix_trsp,y_matrix_trsp, 0,base_vecs-1,0,base_vecs-1,0,base_vecs-1,1,timnum);
   printf("5\n");
   fflush(stdout);

/* Aufraeumen */
   free_matrix(x_matrix_trsp,0,base_vecs-1,1,timnum);
   free_matrix(xx_matrix,0,base_vecs-1,0,base_vecs-1);

/* jetzt wird die Matrix y_matrix_trsp fr spaetere Verwendung weggeschrieben   */
   printf("Storing transformation matrix to file %s\n",transform_file);
   fflush(stdout);

   transform=fopen(transform_file,"wb");
   for (i=0;i<base_vecs-1;i++)
   {
      for (k=1;k<=timnum;k++)
      {
         fwrite(&y_matrix_trsp[i][k],sizeof(real),1,transform);
      }
   }
   fclose(transform);
}
else /*... if (transform == NULL) */
{
   printf("Reading transformation matrix from %s\n",transform_file);
   fflush(stdout);
   y_matrix_trsp = matrix(0,base_vecs-1,1,timnum);

   for (i=0;i<base_vecs-1;i++)
   {
      for (k=1;k<=timnum;k++)
      {
         fread(&y_matrix_trsp[i][k],sizeof(real),1,transform);
      }
   }
   fclose(transform);
} /* endif ... if (transform == NULL) */

/* jetzt an den zu transformierenden Daten weiterarbeiten */

/* Speicherplatz fuer gesamten binaeren Datenanteil reservieren ('mal sehen wie lange es geht...) */
  all_data = matrix(1,oh1.nvars,1,timnum);

/* Speicherplatz fuer einen Satz */
  dat =malloc(oh1.nvars*sizeof(float));

/* Simulationsdaten einlesen */
  printf("Reading data..\n");
  fflush(stdout);
/* Hauptschleife ueber alle Variablen */
  for (k=1;k<=timnum;k++)
     {
/* Einen "Tag" einlesen */
      fread(dat,sizeof(float),oh1.nvars,in_file);

      for (i=1;i<=oh1.nvars;i++)
        {
        all_data[i][k] = (real) dat[i-1];
        }
     }
  fclose(in_file);

/* Vektor zur Aufnahme einer Variablen erzeugen */
    base_data = vector(1,timnum);

    for (i=1;i<=oh1.nvars;i++)
    {
        sprintf(helpname,"%s",strmid(all_var_names,(i-1)*namelength,namelength));
        strtrim(helpname);
    	printf("Transforming %s\n",helpname);
	fflush(stdout);
/* Eine Variable umschaufeln */
       for (k=1;k<=timnum;k++) base_data[k] = all_data[i][k];
/* Mittelwert bestimmen */
       var_coeff[0] = average(base_data,1,timnum);
/* auf diesen verschieben */
       for (k=1;k<=timnum;k++) base_data[k] -= var_coeff[0];
/* und die Harmonischen berechnen */
       matr_vect_mult(y_matrix_trsp,base_data,var_coeff,0,base_vecs-1,1,timnum,1,timnum,1,base_vecs);

/*  wenn die Autokorrelation berechnet werden soll, wird hier die zusätzliche Berechnung gemacht */
  if (function==1)
        {
                target_data[i][0] = var_coeff[0]*var_coeff[0];
                for (j=1;j<=base_vecs;j++) target_data[i][j] = var_coeff[j]*var_coeff[j]/2.0;
        }
        else
        {
        /* sonst wird nur auf die Zieldaten umkopiert */
                for (j=0;j<=base_vecs;j++) target_data[i][j] = var_coeff[j];
        }
  }

/* Header fuer die Ausgabedatei erzeugen */
  oh2.nvars= oh1.nvars;
  oh2.startim  = oh1.startim;
  oh2.endtim   = oh1.endtim;
  oh2.maxdelt  = (float) base_vecs + (2*num_of_cycles-1);

/* outdelt setzen default: wie Input, sonst wie Uebergabewert */
  if (set_outdelt < 0.0)
  {
     oh2.outdelt  = oh1.outdelt;
  }
  else
  {
     oh2.outdelt  = set_outdelt;
  }
  oh2.year     = num_of_cycles;
  oh2.cycle    = oh1.cycle;
  strncpy(oh2.sdir,"fourier     ",12);
  for (m=12;m<namelength;m++) oh2.sdir[m]= (char) 32;
  oh2.sdir[namelength-1]= (char) 0;

  out_file=fopen(output_file,"wb");

 /* Header schreiben */
  writeheader(out_file,&oh2,all_var_names,namelength);

  for (i=1;i<=oh1.nvars;i++) dat[i-1] = (float) target_data[i][0];
  fwrite(&dat[0],sizeof(float),oh2.nvars,out_file);

/* hier wird so mit Nullen aufgefuellt, 
    dass bei Abtastung von mehreren Perioden, mit der richtigen Oberwelle der Abtastperiode angefangen wird  */
  for (i=1;i<=oh1.nvars;i++) dat[i-1] = 0.0;
  for (j=1;j<=2*(num_of_cycles-1);j++)
  {
     fwrite(&dat[0],sizeof(float),oh2.nvars,out_file);
  }
/* ab hier werden nun die Spektralwerte, jetzt mit dem richtigen Oberwellen-Index geschrieben */
  for (j=1;j<=base_vecs;j++)
  {
     for (i=1;i<=oh1.nvars;i++) dat[i-1] = (float) target_data[i][j];
     fwrite(&dat[0],sizeof(float),oh2.nvars,out_file);
  }

  fclose(out_file);

  printf("Ready\n");
  return(0);

}




