#!/bin/sh
# the next line restarts using wish\
exec wish "$0" "$@"
# CEMOS/CEMTK 4.1.1 Copyright(C) 2017 Cora Kohlmeier, Frank Hamberg

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#For details see License Button.

#MoCoS  CEMoS Model  Compare System
global GUI
set GUI 1

global toolflag
global cemhome cemoshome moviehome datahome cemospar netCDFHOME
global pwd home modelhome modelhomeabs model_path
global simname simtype
global allbuts startbuts firstread  log
global DOS

set dirlist "cevutil util mocutil cemos"

foreach item $dirlist {

  auto_mkindex $cemhome/$item *.tcl
  lappend autopath $cemhome/$item

  set dir $cemhome/$item
  set unit [open $cemhome/$item/tclIndex]
  eval [read $unit]
  close $unit
}

# Preparing fileboxes (not necessary but nice for hidden files)
catch { tk_getOpenFile -w00t } ;# start file dialog with unknown option
    set ::tk::dialog::file::showHiddenBtn 1 
    set ::tk::dialog::file::showHiddenVar 0 

##########################################################################
##########################################################################
# Betriebssystem ermitteln
set_arch

##########################################################################
##########################################################################
set toolflag mocos
set home $env(HOME)
set cemhome $env(TKCEMHOME)
set cemoshome $env(CEMOS)
set moviehome $env(TKMOVIEHOME)
set netCDFHOME $cemoshome/netcdf
set pwd [pwd]

set model_path $pwd
if [file exists $pwd/cemos.par] {set cemospar cemos.par}

if {$argc>0} {
 set cemospar [lindex $argv 0]
 if {[file tail $cemospar] == "cemos.par" } {
    set pwd [file dirname $cemospar]
 }
}

if {[catch {package require Scrolledframe}]} \
{
   source [file join $moviehome/general/scrolledframe.tcl]
   package require Scrolledframe
}
namespace import ::scrolledframe::scrolledframe

#erstmal Leichen beseitigen
if [file exists $pwd/integrat.par] {file delete $pwd/integrat.par}
if [file exists $pwd/.sensitivity.par] {file delete $pwd/.sensitivity.par}
if [file exists $pwd/.sensitivity.def] {file delete $pwd/.sensitivity.def}
if [file exists $pwd/.parfile.tmp] {file delete $pwd/.parfile.tmp}

set simname $pwd/result
set modelhomeabs $pwd/main

if ![file exist $home/.cemresource] {exec cp $cemhome/cemresource $home/.cemresource}
option readfile $home/.cemresource

#------------------------------
# Window manager configurations
#------------------------------
wm title . {Tiger Graphics MoCoS}
wm geometry . "+150+50"
wm iconbitmap . @$cemhome/icons/tinylogo.pic
wm iconmask . @$cemhome/icons/tinylogo.pic

#------------------------------
# global settings
#------------------------------
global lfont sfont
global geom xc
global command cancel
global debug bg c2
global editor

global rnum
set rnum 0

set bg 0
set debug 0
set c2 #e8e8ff
set xc #d8d8f8
set defcol [option get . background {}]
set c2 $defcol
set geom +150+0
set cancel 0

readdefaults

# Initializing of the info window
#---------------------------------
execlog

# Model Control
#----------------------------

label .topp.lab -text "MODEL: " -font $lfont -anchor w -bg $c2
pack .topp.lab -side left

label .topp.pwd -text "$pwd" -font $lfont -anchor w -bg $c2
pack .topp.pwd -side left


# Initializing of variables
#---------------------------------
ReadPar
WritePar


#------------------
# Parameter buttons
#------------------
cm_readpar $pwd/mocos/mocos.dat
cm_parbut


# Simulation Control
#----------------------------
frame .sim -bg $c2
pack .sim -fill x -ipady 5


button .sim.sim -text {Start !} -font $lfont  -bg $c2 \
-command {
          if [file exist .sensitivity.def] {exec rm -f .sensitivity.def}
          startbatch $rnum
	  incr rnum
          start_movie 1

         }
pack .sim.sim -side right -padx 10 -pady 10

button .sim.ok -text {Accept} -bg $c2 -command {moc_write_bat}
pack .sim.ok -side right -padx 10 -pady 10


button .sim.res -text {Reset} -bg $c2 -command {moc_reset}
pack .sim.res  -side right -padx 10 -pady 10


#------------------------------
# Set Integration parameters
#------------------------------
moc_integpar
activate_start


set startbuts ".sim.sim"


# Miscs
#------------------
frame .misc -bg $c2
pack .misc -fill x -ipady 15

button  .misc.license -text License -font $lfont \
        -command {deactivate $allbuts;deactivate $startbuts;tatzetool license} -bg $c2
pack  .misc.license -side left
lappend allbuts .misc.license


button  .misc.tatze -bitmap @$cemhome/icons/tinylogo.pic \
        -command {deactivate $allbuts;deactivate $startbuts;tatzetool cemos} -bg $c2
pack  .misc.tatze -side right
lappend allbuts .misc.tatze

button  .misc.exit -text {   Exit  }  -font $lfont -bg $c2\
   -command {destroy .
                        if [file exists $pwd/integrat.par] {file delete $pwd/integrat.par}
                        if [file exists $pwd/.sensitivity.par] {file delete $pwd/.sensitivity.par}
                        if [file exists $pwd/.sensitivity.def] {file delete $pwd/.sensitivity.def}
                        if [file exists $pwd/.parfile.tmp] {file delete $pwd/.parfile.tmp}
            }
pack  .misc.exit  -side right -fill x

button  .misc.movtk -text {Tiger Graphics MoViE} -font $lfont  -bg $c2 \
        -command {start_movie}

pack  .misc.movtk  -side right -fill x


ReadPar
WritePar

deactivate .sim.sim


proc start_movie {{a ""}} {
global pwd rundir
#set files [eval exec ls $pwd/mocos]

set files [glob -nocomplain -directory  $rundir *]
	          set simlist ""
                  foreach item $files {
                    set item [file tail $item]

		     if {[file extension $item]==".outc"} {lappend simlist $rundir/$item}
                  }
	          eval exec mov $simlist $a
}
setvars
