/* MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg */

/* Bestimmen der Inversen einer Matrix mit Gauss-Jordan-Elimination.
Arbeitsweise:
Die regulaere Matrix A wird in dem ersten Durchlauf mit dem Gauss-
Algorithmus in Dreieckform gebracht, wobei alle Operationen parallel
auf die Einheitsmatrix A1 angewandt werden. In einem zweiten Durchlauf
wird dann A diagonalisiert. Auch hier werden alle Operationen parallel
auf A1 angewandt.
Einschraenkungen:
Keine optimale Wahl der Pivot-Elemente.
Autor: Gerhard Krucker
Datum: 28.3.1995
Sprache: MS Visual C V1.5 (QuickWin)
Portiert fuer CEMoS und gcc 3.3, Frank Hamberg 2004
*/

#include "stdinc.h"
#include "constructs.h"

void matr_inversion(real **A,real **out_matrix,int start,int end)
{
/* Input :  real **A			Pointer to input matrix A dimension n
                int n				dimension of input and output matrix

   Output: real **out_matrix	Values are written to out_matrix

   Indices are starting @ 0 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*/

/* A1 wird f�r Zwischenrechnungen gebraucht */
real **A1;
int i,j,k;

A1=matrix(start,end,start,end);

/* A1 mit Einheitsmatrix initialisieren */
for (i=start; i <= end; i++)
   {
   for (j=start; j <= end; j++)
       {
       if (i==j)
          A1[i][j] = 1.0;
       else
          A1[i][j] = 0.0;
       }
   }
/* A in Dreieckform bringen. Parallel alle Operationen auf A1 anwenden */

i=0;

for (k=start; k < end ;k++)
{
   real temp;
   if (A[k][k] == 0) /* Wenn A[k][k] == 0 dann Zeile tauschen */
      {
      i = k+1;
      while ((i <= end) && (A[i][k] == 0)) i++;
      if (i > end) std_error("MATR_INVERSION: System ist singul�r\n");

      for (j=k; j <= end; j++)
         {
         temp = A[i][j]; /* Koeffizient in A vertauschen */
         A[i][j]=A[k][j];
         A[k][j]=temp;
         temp = A1[i][j]; /* Koeffizient in A1 vertauschen */
         A1[i][j]=A1[k][j];
         A1[k][j]=temp;
         }
      }
for (i=k+1; i <=end; i++) /* Gauss Elimination: Fuer jede Zeile ..*/
  {
  for (j=k+1;   j <= end; j++) A[k][j] /= A[k][k]; /* Dividiere die Zeile k durch Akk (Pivot wird = 1) */
  for (j=start; j <= end;j++) A1[k][j] /=A[k][k];
  A[k][k]=1.0;

  for (j=k+1;   j <= end; j++) A[i][j]=A[i][j]-A[i][k]*A[k][j]; /* Fuer einen Teil von A */
  for (j=start; j <= end; j++) A1[i][j]=A1[i][j]-A[i][k]*A1[k][j]; /* Fuer alle Elemente von A1 */
  A[i][k]=0.0;
  }
}

/* Letzte Zeile durch Ann dividieren */
for (j=start; j <= end; j++) A1[end][j] /= A[end][end];
A[end][end]=1.0;

/* Diagonalisieren */
/* A in Diagonalform bringen. Parallel alle Operationen auf A1 anwenden */
for (k=end; k >= start ;k--)
   {
   for (i=k-1; i >= start; i--) /* Gauss Elimination: Fuer jede Zeile ..*/
      {
      for (j=end;  j > k; j--)          A  [i][j]=A[i][j]-A[i][k]*A[k][j];
      for (j=start; j <= end; j++) A1[i][j]=A1[i][j]-A[i][k]*A1[k][j];
      A[i][k]=0.0;
      }
   }
/* Umschaufeln in out_matrix */
for (i=start ; i <= end; i++)
   {
   for (j=start; j <= end;j++) out_matrix[i][j]=A1[i][j];
   }
}
