# MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg

proc compplot {j} {
global can mf
global outdelt timewin
global actstart
global pxmin pxmax pymin pymax pxdelt pydelt
global xmin xmax ymin ymax xdelt ydelt
global colorlist colorindex
global plotFont boldFont
global command
global graph gl
global vertoff
global sameaxis firstaxis calcaxis calcext globalmin globalmax startonzero
global smooth barchart
global filename filelist
global singleresult
global legendtext


#global dataname datalist
global xvgr_flag matlab_flag gnu_flag
global fixymax fixymin xax prettyaxis
global multiset valu spectrum harm numharm what harmend power_spec
global startharm


set c $mf.left.plot 
set sets  [llength $filelist]
set specsets 0
foreach item $filelist {
  if {$what($item)=="fourier"} {incr specsets}
}

set graph [lindex $gl $j]
 
if {$startonzero==1} {set ymin($j) [min $ymin($j) 0]}



if { [regexp {[^0-9\.-]} $fixymin ] == 1 }  {set fixymax ""}

if {$fixymin !="" && ($firstaxis==0 || $j==0)}  {
  set ymin($j) $fixymin
} else {
  if {$startonzero==1} {set ymin($j) [min $ymin($j) 0]}
}

if { [regexp {[^0-9\.-]} $fixymax ] == 1 }  {set fixymax ""}


if {$fixymax !="" && ($firstaxis==0 || $j==0)}  {
  set ymax($j) $fixymax
} else {
  if {$prettyaxis==1} {
   set ymax($j) [max [rd $ymax($j)] 0]
  } else {
   set ymax($j) [max $ymax($j) 0]
  }
}

if {$ymin($j) >= $ymax($j)} {
   set ymax($j) [expr $ymin($j)+1.0]
   if {$ymin($j) > $ymax($j)} {textinfo "ymax<ymin !!"}

}

# Setting of x-axis
#--------------------------
if {$spectrum==0} {
  set xmin $actstart(1)
  set xmax [expr $xmin + $timewin]
  if {$barchart==1} {set xmax [expr $xmax+$outdelt(1)]}
} else {

  set xmin [max 0 [expr (2.0*$startharm-1)*$outdelt(1)]]
  set xmax [max $outdelt(1) [expr (2.0*($harm+$startharm)+1)*$outdelt(1)]]

  if {$startharm>0} { 
      set xmin [max 0 [expr (2.0*$startharm)*$outdelt(1)]]
      set xmax [max $outdelt(1) [expr (2.0*($harm+$startharm))*$outdelt(1)]]
  }
#  set xmax [min $xmax [expr (2.0*($harmend)+2)*$outdelt(1)]]
#   set xmax [expr $xmin +($harm*2+1)*$outdelt(1)]
}

set xdelt [expr $xmax -$xmin]
set ydelt [expr $ymax($j) - $ymin($j)]

# Plotting of axes
#--------------------------

axes $j

# Setting of legendtext
#----------------------

make_legendtext $j

 
set header [makeheader [lindex $command($j) 0]]

if {$xvgr_flag == 1} {
   xv_open_graph $j $actstart(1) [expr $actstart(1)+$timewin] \
                                      $ymin($j) $ymax($j) $header xx
}


if {$matlab_flag == 1} {
   matlab_open_graph $j $actstart(1) [expr $actstart(1)+$timewin] \
                                      $ymin($j) $ymax($j) $header xx
}


if {$gnu_flag == 1} {
   gnu_open_graph $j $actstart(1) [expr $actstart(1)+$timewin] \
                                      $ymin($j) $ymax($j) $header xx
}


set colorindex 0
set setnum 0

foreach selectvar $command($j) {
  
  if {$multiset==0} {
	set color black
  } else {
    set color [lindex $colorlist $colorindex]
  }
  # Plotting of variable name
  #--------------------------
  set varrow [expr 20+$vertoff*$colorindex]
  set header [makeheader $selectvar]
  set item [$c create text 100 $varrow -text $header -fill $color \
          -font $plotFont -anchor w]
  $c addtag axis withtag $item
  $c addtag $graph withtag $item

  set plotset 0

  foreach k $filelist {

   # Plotting of legend
   #-------------------
   if {$multiset==0} {
     set filecolor [lindex $colorlist [expr $k-1]]
     set color $filecolor
     set width 1
   } else {
     set filecolor black
     set width [expr [llength $filelist] -$k +1]
   }
   set row [expr 20+$vertoff*[expr $k-1]]

   if {$j == 0} {
    if {$multiset==1} {
      set item [$c create line 400 $row 440 $row -width $width ]
      $c addtag $color$graph withtag $item
      $c addtag legend$graph withtag $item
      $c addtag $graph withtag $item
    }

    set item [$c create text 450 $row -text $legendtext($k) \
             -font $plotFont -fill $filecolor -anchor w]
    $c addtag $color$graph withtag $item
    $c addtag legend$graph withtag $item
    $c addtag $graph withtag $item

    $c bind legend$graph <1> "complegendDown $c %x %y legend$graph;\
                           $c itemconfig legend$graph -font {$boldFont}"
    $c bind legend$graph <ButtonRelease-1> "complegendMove $c %x %y;\
                           $c dtag selected;\
                           $c itemconfig legend$graph -font {$plotFont}"
    set compleg(lastX) 0
    set compleg(lastY) 0
   }

  # Preparing data
  #-----------------
   set dat $singleresult($k,$selectvar)

   if [info exist plotdata] {unset plotdata}
   set px [nnx $xmin]
   set poutdelt [expr $outdelt($k)*(1.0*$pxdelt)/$xdelt]

   if {$power_spec==0} {
                foreach y $dat {
                lappend plotdata $px [nnyj $y $j]
                set px [expr $px+$poutdelt]
                }
        } else {
                for {set ip 0} {$ip < [expr [llength $dat]/2]} {incr ip} {
                                if {$ip==0} { 
                                        set value1 [lindex $dat $ip]
                                        set value1 [expr $value1*$value1]
                                        set value2 0
                                } else {
                                        set value1 [lindex $dat [expr 2*$ip-1]]
                                        set value1 [expr $value1*$value1]
                                        set value2 [lindex $dat [expr 2*$ip]]
                                        set value2 [expr $value2*$value2]
                                }
                                lappend plotdata $px [nnyj [expr $value1+$value2] $j]
                                set px [expr $px+2.0*$poutdelt]
                        }
     }
  # Plotting one data set
  #------------------------

  if {$barchart ==0} {

     if {$smooth==1} {set sm true} else {set sm false}
     
     myplot $c $plotdata $width $color $sm $graph

   } else {

       set anfang 0
       set ende [llength $plotdata]

     if {$spectrum==0} {
        set xoffset [expr $poutdelt/$sets]
	set plotset $k
     } else {

        set xoffset [expr $poutdelt/$specsets]

        if {$what($k)!="fourier"} {
	  textinfo "No spectral values  available for [file tail $filename($k)] !"
	} else {
	  incr plotset
	}
     }

     if {$k==1} {set fillcolor grey} else {set fillcolor $color}

     for {set i $anfang} {$i <$ende} {set i [expr $i+2]} {

       set x1 [expr [lindex $plotdata $i]+ [expr $plotset-1]*$xoffset]
       set y1 [lindex $plotdata [expr $i+1]]
       set x2 [expr $x1+$xoffset]
       set y2 [nnyj 0 $j]


       if {$spectrum==0 || $what($k)=="fourier"} {

         set item [eval {$c create rect} $x1 $y1 $x2 $y2 -width 1 -fill $fillcolor ]
	 $c addtag $color$graph withtag $item
         $c addtag $graph withtag $item
       }

     }
   }

   $c bind $color$graph <Control-Button-3> "set eraseset 1;makecommand -1 -1 $colorindex $j"


  if {$xvgr_flag ==1 } {
     set xvdat $singleresult($k,$selectvar)
     xv_set_num $j [expr $k-1] $xvdat $legendtext($k) $outdelt($k)
  }


  if {$matlab_flag ==1 } {
     set matlabdat $singleresult($k,$selectvar)
     matlab_set_num $j [expr $k-1] $matlabdat $legendtext($k) $outdelt($k)
  }

    if {$gnu_flag ==1 } {
       set gnudat $singleresult($k,$selectvar)
       gnu_set_num $j [expr $k-1]  $gnudat $legendtext($k) $outdelt($k)
    }

}
incr colorindex
}

if {$valu==1} {valuplot $j $row $k}

if {$matlab_flag ==1 } {
   matlab_close_graph 
}

if {$gnu_flag ==1 } {
   gnu_close_graph 
}

}


