# MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg

proc pk_distances {} {
global what command cycle numharm result distance filename 
global distvar distval distnum
global gc distharm
global distshow ttt
global starttime endtime

if ![info exist filename(2)] {
  $ttt insert end  "2nd Result-File missing!\n"
  $ttt insert end  "No L2-Distance is calculated!\n"
 return
}

if [info exist filename(3)] {
  $ttt insert end  "3rd and higher  Result-Files ignored.\n"
  $ttt insert end  "L2-Distance is calculated between File 1 and 2!\n"
}

if {$what(1)!="fourier" || $what(2) != "fourier"} {
   $ttt insert end  "One of the two files does not hold Fourier coefficients\n"
   $ttt insert end  "No L2-Distance is calculated!\n"
return
}

if {$cycle(1)!=$cycle(2)} {
   $ttt insert end  "Basic cycles differ!\n"
   $ttt insert end  "Cycle(1)=$cycle(1)\n"
   $ttt insert end  "Cycle(2)=$cycle(2)\n"
   $ttt insert end  "No L2-Distance is calculated!\n"
return
}

if {$starttime(1)!=$starttime(2)} {
   $ttt insert end  "Starttimes differ!\n"
   $ttt insert end  "Starttime(1)=$starttime(1)\n"
   $ttt insert end  "Starttime(2)=$starttime(2)\n"
   $ttt insert end  "No L2-Distance is calculated!\n"
 return
}

if {$endtime(1)!=$endtime(2)} {
   $ttt insert end  "Endtimes differ:\n"
   $ttt insert end  "Endtime(1)=$endtime(1)\n"
   $ttt insert end  "Endtime(2)=$endtime(2)\n"
   $ttt insert end  "No L2-Distance is calculated!\n"
   return
}

set distharm [min $numharm(1) $numharm(2)]
if {$numharm(1)!=$numharm(2)} {
   $ttt insert end  "Number of harmonics differ.\n"
   $ttt insert end  "Distance calculated for $distharm harmonics.\n"
}

set distvar ""
for {set j 0} {$j<=$distnum} {incr j} {
  set distval($j) ""
}

for {set i 0} {$i<=$gc} {incr i} {

  set distance 1
  set selectvar [lindex $command($i) 0]
  lappend distvar $selectvar

  extract_result 1
  set f $result($selectvar)

  extract_result 2
  set g $result($selectvar)
  set distance 0

 for {set j 0} {$j<=9} {incr j} {
  if {$distshow($j)==1} {

     #lappend distval($j) [format "%6.3f" [expr abs([pk_distance$j $f $g])]]

     set actdist [pk_distance$j $f $g]
   
     if {$actdist !="undef"} {
         lappend distval($j) [format "%6.3f" $actdist]
     } else {
          lappend distval($j) undef
     }
 
      
     }
   }
}

distview
}

#
#...................................................................................
#

proc l2norm {f} {
global distharm

set norm 0
for {set l 0} {$l < $distharm} {incr l} {
    set el [lindex $f $l]
    set norm [expr $norm + $el*$el]
}
return [expr sqrt($norm)]
}
#
#...................................................................................
#
proc skalar {f g} {
global distharm

set skprd 0
for {set l 0} {$l < $distharm} {incr l} {
    set fl [lindex $f $l]
    set gl [lindex $g $l]
    set skprd [expr $skprd + $fl*$gl]
}
return $skprd
}

#
#...................................................................................
#
# distance ||f-g||
proc pk_distance0 {f g} {
global distharm

for {set i 0} {$i < $distharm} {incr i} {
  lappend fminusg [expr [lindex $f $i]-[lindex $g $i]]
}

set dist [l2norm $fminusg]

return $dist
}
#
#...................................................................................
#
# distance ||f-g||/||f+g||
proc pk_distance1 {f g} {
global distharm

for {set i 0} {$i < $distharm} {incr i} {
  lappend fplusg [expr [lindex $f $i]+[lindex $g $i]]
  lappend fminusg [expr [lindex $f $i]-[lindex $g $i]]
}

set nenner [l2norm $fplusg]

if {$nenner > 0} {
   set zaehler [l2norm $fminusg]
   set dist [expr $zaehler/$nenner]
} else {
   set dist infinity
}
return $dist
}
#
#...................................................................................
#
# distance ||f-g||/min(||g||,||f||)
proc pk_distance2 {f g} {
global distharm

for {set i 0} {$i < $distharm} {incr i} {
  lappend fminusg [expr [lindex $f $i]-[lindex $g $i]]
}

set nenner [min [l2norm $f] [l2norm $g]]

if {$nenner > 0} {
   set zaehler [l2norm $fminusg]
   set dist [expr $zaehler/$nenner]
} else {
   set dist infinity
}
return $dist
}
#
#...................................................................................
#
# distance ||f-g||/(||g||+||f||-||f-g||)
proc pk_distance3 {f g} {
global distharm

for {set i 0} {$i < $distharm} {incr i} {
  lappend fminusg [expr [lindex $f $i]-[lindex $g $i]]
}
set normfminusg [l2norm $fminusg]
set nenner [expr [l2norm $f] + [l2norm $g] - $normfminusg]

if {$nenner > 0} {
   set zaehler $normfminusg
   set dist [expr $zaehler/$nenner]
} else {
   set dist infinity
}
return $dist
}
#
#...................................................................................
#
#distance 1-<g,f>/(||g||*||f||)
proc pk_distance4 {f g} {
global distharm


set nenner [expr [l2norm $f]*[l2norm $g]]

if {$nenner > 0} {
   set zaehler [skalar $f $g]
   set dist [expr 1.0 - $zaehler/$nenner]
} else {
   set dist infinity
}
return $dist
}
#
#...................................................................................
#
#distance (||g||*||f|-<g,f>)/(||g||*||f||+<g,f>)
proc pk_distance5 {f g} {
global distharm

set produkt [expr [l2norm $f]*[l2norm $g]]
set skalarprodukt [skalar $f $g]

set nenner [expr $produkt + $skalarprodukt]

if {$nenner > 0} {
   set zaehler [expr $produkt - $skalarprodukt]
   set dist [expr $zaehler/$nenner]
} else {
   set dist infinity
}
return $dist
}
#
#...................................................................................
#
#distance arccos(<g,f>/(||g||*||f||)
proc pk_distance6 {f g} {
global distharm

set nenner [expr [l2norm $f]*[l2norm $g]]

if {$nenner > 0} {
   set zaehler [skalar $f $g]
   set dist [expr $zaehler/$nenner]
   set dist [expr acos ([min 1.0 [max -1.0 $dist]])]
} else {
   set dist undef
}
return $dist
}
#
#...................................................................................
#
#distance |ln(||g||/||f||)|
proc pk_distance7 {f g} {
global distharm

set nenner [l2norm $f]

if {$nenner > 0} {
   set zaehler [l2norm $g]
   set argu [expr $zaehler/$nenner]
   set dist [expr abs(log ($argu))]
} else {
   set dist infinity
}
return $dist
}
#
#...................................................................................
#
#distance max(||g||/||f||,||f||/||g||)-1
proc pk_distance8 {f g} {
global distharm

set normf [l2norm $f]
set normg [l2norm $g]

if {$normf > 0 && $normg >0} {
  set argu1 [expr $normf/$normg]
  set argu2 [expr $normg/$normf]
   set dist [expr [max  $argu1 $argu2] -1.0]
} else {
   set dist infinity
}
return $dist
}
#
#...................................................................................
#
#distance 9 = d6 + d7
proc pk_distance9 {f g} {

if { [pk_distance6 $f $g] !="undef" &&  [pk_distance7 $f $g] != "infinity" } {
   set dist [expr [pk_distance6 $f $g] + [pk_distance7 $f $g]]
} else {
   set dist undef
}
return $dist
}
#
#...................................................................................
#



