# MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg

proc cont_plot {{c none}} {

global can mf
global starttime endtime outdelt year cycle timewin
global actstart actend
global pxmin pxmax pymin pymax pxdelt pydelt
global xmin xmax ymin ymax xdelt ydelt
global filmdata
global selectvar
global map actbox animcolor
global filmcolor colmax colmin filmmin filmmax
global mapcoor
global plotFont boldFont
global command
global startbox boxwindow first last layersum layer
global markzeros layervalue evallayer tiefe

if {$c == "none"} {set c $mf.left.plot}
if [info exist data] {unset data}

set colrange [expr $colmax-$colmin]

set fmax [lindex $filmdata($startbox) 0]
set fmin $fmax

set lastbox [expr $startbox + $boxwindow]
for {set i $startbox} {$i <= $lastbox} {incr i} {
  set data($i) [lrange $filmdata($i) $first $last]
  set fmax [max $fmax [lmax $data($i)]]
  set fmin [min $fmin [lmin $data($i)]]

}

if {$filmmin != ""} {set fmin $filmmin}
if {$filmmax != ""} {set fmax $filmmax}

set frange [expr $fmax -$fmin]

if {[expr abs($frange)] <= [expr abs($fmax*1e-10)]} then {set frange 1}

set fac [expr double($colrange)/double($frange)]

for {set i $startbox} {$i <= $lastbox} {incr i} {
  set animcolor($i) {}
  
  foreach item $data($i) {
   scan [expr $colmax-($item-$fmin)*$fac] %d help
   
   set help [max $help $colmin]
   set help [min $help $colmax]

   if {[expr abs($item)] < 0.00000001 && $markzeros==1} {
       lappend animcolor($i) grey
   } else {
       lappend animcolor($i) $filmcolor($help)
   }
  }
}

# erasing old
#------------
erase $c all

# Writing header
#--------------------------
regsub -all {[(][0-9]+[)]} $selectvar "" selectvarname

set item [$c create text $pxmin 30 -text [makeheader $selectvarname] \
                   -anchor w -font $boldFont]
$c addtag head withtag $item


# Setting up of the colorbar
#--------------------------
set pxl [expr $can/2]
set pxl $pxmin
set pyu [expr $pymin+70]
set pyo [expr $pymin+50]
set remx $pxl

set offset [expr 1.0*($pxmax-$pxmin)/$colrange]

for {set i $colmax} {$i >= $colmin} {set i [expr $i-1]} {
    set pxr [expr $pxl+$offset]
    set item [eval $c create rect $pxl $pyu $pxr $pyo \
                    {-fill $filmcolor($i) -outline $filmcolor($i)}]
    $c addtag bar withtag $item
    set pxl $pxr
}

set item [$c create text $remx [expr $pyu+10] -text $fmin \
                   -anchor w -font $boldFont]
$c addtag bar withtag $item
set item [$c create text $pxl [expr $pyu+10] -text $fmax \
                   -anchor e -font $boldFont]
$c addtag bar withtag $item

# Plotting the colordata
#--------------------------

   set ymin 0
   set ymax $layersum
   set ydelt $layersum

   set xmin $first
   set xmax $last
   set xdelt [expr 1.0*($last -$first)+1]

   set x1 $xmin


   for {set i $first} {$i<=$last} {incr i} {

      set y1 $ymax

       for {set j $startbox} {$j<=$lastbox} {incr j} {

	 set x2 [expr 1.0*($x1+1)]
         set y2 [expr 1.0*($y1-$layer($j,$i))]

         set color [lindex $animcolor($j) [expr $i-$first]]
	 
	 # wenn color == "" wird nicht gemalt 

         set item [eval {$c create rect} [nnx $x1] [nny $y1] [nnx $x2] [nny $y2] \
                     {-width 1 -fill $color -outline $color }]

	 set y1 $y2
      }
      set x1 $x2
   }

# Plotting "x-axis"

set item [$c create text $pxmin [expr $pymin+30] -text $actstart(1) \
                   -anchor w -font $boldFont]

set item [$c create text $pxmax [expr $pymin+30] -text [expr $actstart(1)+$timewin] \
                   -anchor e -font $boldFont]

set item [$c create text [expr $can/2] [expr $pymin+30] -text Time \
                   -anchor c -font $boldFont]

# Plotting "y-axis"
if {$layervalue==1} {
    set lastbox [expr $startbox + $boxwindow]
    set labelpos $ymax

  for {set i $startbox} {$i <= $lastbox} {incr i} {
    if {$evallayer==1} {
       set ytext  [string trim [format "%10.5g" $tiefe($i)]]
    } else {
       set ytext  [string trim [format "%10.5g" [expr $i]]]
    }

    set ylabelpos [nny $labelpos]


    set item [$c create text [expr $pxmin-5] $ylabelpos -text $ytext \
                   -anchor e -font  $plotFont]

    $c addtag axis withtag $item
    set labelpos [expr $labelpos-$layer($i,$first)]
}

}
}
