#!/bin/sh
# the next line restarts using wish\
exec wish "$0" ${1+"$@"}

# MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#For details see License Button.

# Control for starting mov with tixwish or not
global movieversion
###############################################
set movieversion 5.0
###############################################
global moviehome iconpath datahome home
global pwd datadir
global filenamelist filenum toolflag forcecompare
global DOS
global resultdir tempdir
global defaulticonsize
global onewindow
global initmov
set onewindow 1
set initmov 1
##########################################################################
# Betriebssystem ermitteln
set DOS 0
set arch [exec uname -s]
if {[string match CYGWIN* $arch]==1} {set DOS 1}
if {[string match Darwin* $arch]==1} {set DOS 2}
#puts $DOS

##########################################################################
# Preparing fileboxes (not necessary but nice for hidden files)
if {$DOS==0} {
   catch { tk_getOpenFile -w00t } ;# start file dialog with unknown option
       set ::tk::dialog::file::showHiddenBtn 1 
       set ::tk::dialog::file::showHiddenVar 0
} 
##########################################################################
package require Img
set pwd [pwd]

set moviehome [file dir [info script]]
# für Platypus-Compilate
if {[file tail $moviehome] != "mov"} {set moviehome $moviehome/mov} 
#set moviehome $env(TKMOVIEHOME)
set home $env(HOME)
set datahome $env(HOME)/data

#Scrolledframe einbinden
if {[catch {package require Scrolledframe}]} \
{
   source [file join $moviehome/general/scrolledframe.tcl]
   package require Scrolledframe
}
namespace import ::scrolledframe::scrolledframe

set filenum -1
if {[string match {[0-7]} [lindex $argv end]]} {set tooltypeset 1} else {set tooltypeset 0}
if {$argc>0} {
  set filenamelist [lrange $argv 0  [expr $argc - $tooltypeset -1]]
#  if {[lindex $argv 0] != $tooltype} {set pwd [file dir [lindex $argv 0]]}
  if {![string match {[0-9]} [lindex $argv 0] ]} {set pwd [file dir [lindex $argv 0]]}
} else {
  set filenamelist ""
}
if {[llength $filenamelist] >=2} {set forcecompare 1; set tooltype 1} else {set forcecompare 0;set tooltype -1}

if {$pwd=="."} {set pwd [pwd]}
#if ![file exist $home/.mov5resource] {puts copying;exec cp $moviehome/mov5resource $home/.mov5resource}
#option readfile $home/.mov5resource

set dirlist "general control compare overview text film phase custom prepare profile contour profcom"

foreach item $dirlist {

  auto_mkindex $moviehome/$item *.tcl
  lappend autopath $moviehome/$item

  set dir $moviehome/$item
  set unit [open $moviehome/$item/tclIndex]
  eval [read $unit]
  close $unit
}

# Global definitions
#-------------------

if ![file exist $home/.config/tkmovie5defaults] {
	file copy $moviehome/tkmovie5defaults $home/.config
}

if ![file exist $home/.tkmovierc] {
	file copy $moviehome/tkmovierc $home/.tkmovierc 
}
set tempdir $pwd/.tmp
if ![file exist $tempdir] {
	file mkdir $tempdir 
}

setvars
readdefaults
readnameinfo

if {$defaulticonsize==""} {set defaulticonsize normal}
if {$defaulticonsize == "normal"} {
    set iconpath $moviehome/icons
} else {
    set iconpath $moviehome/icons_128
}
#------------------------------
# Window manager configurations
#------------------------------
wm geometry . "+100+50"
wm iconmask . @$iconpath/tinylogo.pic
wm iconbitmap . @$iconpath/tinylogo.pic
wm protocol . WM_DELETE_WINDOW {.top.exit invoke}

set wx 0
set wy 0
global height width
set height 0
set width 0

bind . <Configure> {
    if {"%W" eq [winfo toplevel %W]} {
        if {($height != %h) || ($width != %w)} {
			set height %h
			set width %w
		}
	}
}


if {$onewindow==1} {
#	set width [expr [winfo screenwidth .] -400] 
#	set height [expr [winfo screenheight .] -250] 
	wm geometry . "+50+50"
#	wm geometry .ov "$width\x$height+50+50"
} else {
	wm geometry . "+50+50"
}

switch $tooltype {
    -1 { }
    0 { set toolflag overview}
    1 { set toolflag compare}
    2 { set toolflag film}
    3 { set toolflag text}
    4 { set toolflag profile}
    5 { set toolflag profcom}
    6 { set toolflag phase}
    7 { set toolflag contour}
}   
tools
createcontrol

