# MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg

proc plot {j {c none}} {

global can mf
global outdelt timewin
global actstart
global pxmin pxmax pymin pymax pxdelt pydelt
global xmin xmax ymin ymax xdelt ydelt
global colorlist colorindex
global plotFont boldFont
global command
global graph gl singleresult
global vertoff
global xvgr_flag gnu_flag matlab_flag
global smooth barchart
global fixymax fixymin
global xax prettyaxis firstaxis startonzero
global not_stored
global filename longnames

if {$c=="none"} {set c $mf.left.plot} 

set graph [lindex $gl $j]

if {$graph=="zero"} {
      if {$longnames==0} {
       set item [$c create text 590 10 -text [file rootname [file tail $filename(1)]] \
		  -font $plotFont -fill black -anchor e]
      } else {
        set item [$c create text 590 10 -text $filename(1) \
	  	  -font $plotFont -fill black -anchor e]
     }
	 $c addtag file withtag $item
	 $c addtag zero withtag $item
}

set init 1
if { [regexp {[^0-9\.-]} $fixymin ] == 1 }  {set fixymax ""}

if {$fixymin !="" && ($firstaxis==0 || $j==0)}  {
  set ymin($j) $fixymin
} else {
 if {$startonzero==1}  {set ymin($j) [min $ymin($j) 0]}
}

if { [regexp {[^0-9\.-]} $fixymax ] == 1 }  {set fixymax ""}

if {$fixymax !="" && ($firstaxis==0 || $j==0)}  {
  set ymax($j) $fixymax
} else {
  if {$prettyaxis==1} {
   set ymax($j) [max [rd $ymax($j)] 0]
  } else {
   set ymax($j) [max $ymax($j) 0]
  }
}

if {$ymin($j) >= $ymax($j)} {
   set ymax($j) [expr $ymin($j)+1.0]
   if {$ymin($j) > $ymax($j)} {textinfo "ymax<ymin !!"}

}

set setnum 0
foreach selectvar $command($j) {

if { $init == 1} {

   set init 0
   set colorindex 0
   set setnum 0

   set xmin $actstart(1)
   set xmax [expr $xmin + $timewin]
   if {$barchart==1} {set xmax [expr $xmax+$outdelt(1)]}
   set xdelt [expr {$xmax - $xmin}]
   set ydelt [expr $ymax($j) - $ymin($j)]



# Plotting of axes
#--------------------------

   axes $j $c
   set title [file rootname [file tail $filename(1)]]

   if {$xvgr_flag == 1} { xv_open_graph $j \
                                     $actstart(1) \
                                     [expr $actstart(1)+$timewin] \
                                      $ymin($j) $ymax($j) $title xx
                        }


   if {$matlab_flag == 1} {matlab_open_graph $j \
                                     $actstart(1) \
                                     [expr $actstart(1)+$timewin] \
                                      $ymin($j) $ymax($j) $title xx

                        }

  if {$gnu_flag == 1} {gnu_open_graph $j \
                                     $actstart(1) \
                                     [expr $actstart(1)+$timewin] \
                                      $ymin($j) $ymax($j) $title xx

                        }

} else {set colorindex [expr $colorindex+1]}

# Plotting of singleresult
#--------------------------
set color [lindex $colorlist $colorindex]
set row [expr 20+$vertoff*$colorindex]

if {$not_stored($selectvar)==0} {
    set header [makeheader $selectvar]
} else {
    set header "$selectvar NOT STORED !!!"
}

set item [$c create text 200 $row -text $header -font $plotFont -fill $color -anchor w]
$c addtag $color$graph withtag $item
$c addtag legend$graph withtag $item
$c addtag $graph withtag $item

$c bind legend$graph <Button-1> "legendDown $c %x %y legend$graph;\
                                 $c itemconfig legend$graph -font {$boldFont}"
$c bind legend$graph <ButtonRelease-1> "legendMove $c %x %y;\
                                        $c dtag selected;\
                                        $c itemconfig legend$graph -font {$plotFont}"

set leg(lastX) 0
set leg(lastY) 0

set sets [llength $command($j)]

# Preparing data for plot
#------------------------
if [info exist plotdata] {unset plotdata}
set px [nnx $xmin]
set poutdelt [expr $outdelt(1)*$pxdelt/$xdelt]

foreach y $singleresult($selectvar) {
    lappend plotdata $px [nnyj $y $j]
    set px [expr $px+$poutdelt]
}


# Plotting one data set
#------------------------
if {$smooth ==1} {set sm true} else {set sm false}

if {$barchart ==0} {

       myplot $c $plotdata 1 $color $sm $graph

} else {

    set xoffset [expr $poutdelt/$sets]

    for {set i 0} {$i < [llength $plotdata]} {set i [expr $i+2]} {

       set x1 [expr [lindex $plotdata $i]+ $setnum*$xoffset]
       set y1 [lindex $plotdata [expr $i+1]]
       set x2 [expr $x1+$xoffset]
       set y2 [nnyj 0 $j]

       if {$colorindex==0} {set fillcolor grey} else {set fillcolor $color}
       set item [eval {$c create rect} $x1 $y1 $x2 $y2 -width 2 -fill $fillcolor ]
       $c addtag $color$graph withtag $item
       $c addtag $graph withtag $item

    }
}
    $c bind $color$graph <Control-Button-3> "set eraseset 1;makecommand -1 -1 $colorindex $j"

    if {$xvgr_flag ==1 } {
       xv_set_num $j $setnum $singleresult($selectvar) $header $outdelt(1)
      # incr setnum
    }

    if {$matlab_flag ==1 } {
       matlab_set_num $j $setnum $singleresult($selectvar) $header $outdelt(1)
      # incr setnum
    }

    if {$gnu_flag ==1 } {
        gnu_set_num $j $setnum $singleresult($selectvar) $header $outdelt(1)
    }

incr setnum
}
 if {$matlab_flag ==1 } {
   matlab_close_graph $setnum
}
 if {$gnu_flag ==1 } {
   gnu_close_graph 
}

}



