# MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg

proc phaseplot {j} {

global can mf
global starttime outdelt timewin
global actstart
global pxmin pxmax pymin pymax pxdelt pydelt
global xmin xmax ymin ymax xdelt ydelt
global colorlist colorindex
global plotFont boldFont
global result command
global graph gl singleresult
global vertoff
global phasescale calcext startonzero
global yminsel ymaxsel
global xvgr_flag matlab_flag gnu_flag
global points
global seq seq_min seq_max
global coltab colrange colmin colmax filmcolor
global blue_ymin blue_ymax

set c $mf.left.plot  

set graph [lindex $gl $j]
set color black


set coltab "phase"
makefilmcolor
set colrange [expr $colmax-$colmin]

set localFont -*-Helvetica-Bold-R-Normal--*-180-*-*-*-*-*-*


if [info exist ymin] {unset ymin;unset ymax}

scan [expr ($actstart(1)-$starttime(1))/$outdelt(1)] %d first
scan [expr $first +$timewin/$outdelt(1)] %d last

set xvar [lindex $command($j) 0]
set yvar [lindex $command($j) 1]

if {$phasescale==1} {set calcext 1}
foreach selectvar $command($j) {
   
   #set singleresult($selectvar) [lrange $result($selectvar) $first $last]

  if {$calcext==1} {
     # Calculating of minimum and maximum of the singleresult
     #--------------------------
     if {$phasescale==1} {
       set ymaxsel($selectvar) [lmax $singleresult($selectvar)]
       set yminsel($selectvar) [lmin $singleresult($selectvar)]
      } else {
       if {$seq==0} {
          set ymaxsel($selectvar) [lmax $result($selectvar)]
          set yminsel($selectvar) [lmin $result($selectvar)]
       } else {
          set ymaxsel($selectvar) $seq_max($selectvar)
	  set yminsel($selectvar) $seq_min($selectvar)
       }
      }


  }
}

   # Setting of axes

   set xmin $yminsel($xvar)
   set xmax $ymaxsel($xvar)
   set ymin $yminsel($yvar)
   set ymax $ymaxsel($yvar)

   if {$startonzero==1} {
      set xmin 0
      set ymin 0
   }

   if {$xmin == $xmax} { set xmax [expr $xmin+1]}
   if {$ymin == $ymax} { set ymax [expr $ymin+1]}

   set xdelt [expr $xmax - $xmin]
   set ydelt [expr $ymax - $ymin]
 
   # Plotting of x-axis and y axis
   #--------------------------

   set item [$c create line $pxmin $pymin $pxmin $pymax -width 1.5]
   $c addtag axis withtag $item
   $c addtag $graph withtag $item

   set item [$c create line $pxmin $pymin $pxmax $pymin -width 1.5]
   $c addtag axis withtag $item
   $c addtag $graph withtag $item

  # Plotting of x-axis labels
  #--------------------------

  for {set i 0} {$i <= 6} {incr i} {
    set xlabel [expr $xmin + ($i*$xdelt/6)]
    set xtext [format "%10.3g" $xlabel]
    set xoff [nnx $xlabel]

   set item [$c create line $xoff $pymin $xoff [expr $pymin+5] -width 2]
   $c addtag axis withtag $item
   $c addtag $graph withtag $item
   set item [$c create text $xoff [expr $pymin+10] -text $xtext \
                   -anchor n -font   $plotFont]
   $c addtag axis withtag $item
   $c addtag $graph withtag $item
  }

  # Plotting of y-axis labels
  #--------------------------

  for {set i 0} {$i <= 6} {incr i} {
    set ylabel [expr $ymin + ($i*$ydelt/6)]
    set ytext [format "%10.3g" $ylabel]
    set yoff [nny $ylabel]

    set item [$c create line [expr $pxmin-5] $yoff $pxmin $yoff -width 2]
    $c addtag axis withtag $item
    $c addtag $graph withtag $item
    set item [$c create text [expr $pxmin-10] $yoff -text $ytext \
                   -anchor e -font $plotFont]
    $c addtag axis withtag $item
    $c addtag $graph withtag $item
  }


# Plotting of legend
#--------------------------


set item [$c create text 200 20 -text "x-axis: $xvar" \
             -font $plotFont -fill $color -anchor w]
    $c addtag $color$graph withtag $item
    $c addtag legend$graph withtag $item
    $c addtag $graph withtag $item

set row [expr 20+$vertoff]

set item [$c create text 200 $row -text "y-axis: $yvar" \
             -font $plotFont -fill $color -anchor w]
    $c addtag $color$graph withtag $item
    $c addtag legend$graph withtag $item
    $c addtag $graph withtag $item



$c bind legend$graph <1> "legendDown $c %x %y legend$graph;\
                           $c itemconfig legend$graph -font {$boldFont}
"
$c bind legend$graph <ButtonRelease-1> "legendMove $c %x %y;\
                           $c dtag selected;\
                           $c itemconfig legend$graph -font {$plotFont}"


set leg(lastX) 0
set leg(lastY) 0

#only for printout
set headx [makeheader $xvar]
set heady [makeheader $yvar]
set header "x-y: $headx - $heady"

   if {$xvgr_flag == 1} { 
     xv_open_graph $j $xmin $xmax $ymin $ymax $header xx
   }

   if {$matlab_flag == 1} { 
     matlab_open_graph $j $xmin $xmax $ymin $ymax $header xx
   }

   if {$gnu_flag == 1} { 
     gnu_open_graph $j $xmin $xmax $ymin $ymax $header xx
   }


# Preparing data for plot
#------------------------
if [info exist plotdata] {unset plotdata}
if [info exist plotcolor] {unset plotcolor}


set len [llength $singleresult($yvar)]
set fac [expr double($colrange)/double($len)]

for {set l 0} {$l< $len} {incr l} {

  set x($l) [nnx [lindex $singleresult($xvar) $l]]
  set y($l) [nny [lindex $singleresult($yvar) $l]]
  lappend plotdata $x($l) $y($l)

  scan [expr $colmax-$l*$fac] %d help
  set help [min [max $help $colmin] $colmax]

  set plotcolor($l) $filmcolor($help)


}


# Plotting of data
#------------------------

if {$points==0} {
  set item [eval {$c create line} $plotdata {-width 1 -fill $color -arrow last}]
  $c addtag $color$graph withtag $item
  $c addtag $graph withtag $item
} else {

  for {set l 0} {$l< $len} {incr l} {

    set item [eval {$c create text $x($l) $y($l) -fill $plotcolor($l) -text "*"}]

   if {$l==0} { set item [eval {$c create text $x($l) $y($l) -fill green -text "*" -font $localFont }]}
   if {$l==[expr $len-1]} { set item [eval {$c create text $x($l) $y($l) -fill red -text "*" -font $localFont }]}


    $c addtag $color$graph withtag $item
    $c addtag $graph withtag $item
  }
}



if {$xvgr_flag ==1 } {
       xv_set_num $j 0 $singleresult($yvar) "" $outdelt(1) $singleresult($xvar)
}

if {$matlab_flag ==1 } {
       matlab_set_num $j 0 $singleresult($yvar) "" $outdelt(1) $singleresult($xvar)
}

if {$gnu_flag ==1 } {
       gnu_set_num $j 0 $singleresult($yvar) "" $outdelt(1) $singleresult($xvar)
}

set blue_ymin $ymin
set blue_ymax $ymax

if [info exist ymin] {
set yminrem $ymin
set ymaxrem $ymax
unset ymin
unset ymax

set ymin($j) $yminrem
set ymax($j) $ymaxrem

}

if {$gnu_flag ==1 } {
   gnu_close_graph 
}

}



